%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
% -------------------------------------------------------------------------

% --------------------------------------------------------------------- %%
%                          Load Inputs                                   %
% ---------------------------------------------------------------------  %   
% load('CaseStudy_I/20180627/Results_complete.mat');
load('CaseStudy_I/Inputs_CaseStudy_I.mat');

% Load list of fluids
ListOfFluids;

% Define best cases:
best_solutions = [
                  20 11   1 2   % Ammonia
                   9 10   1 2   % Butane
                  16 11   1 2   % R1234ze(Z)
                   4  7 0.2 2   % 20%CO2 / 80 % DME
                   5 17 0.3 2   % 30% Propylene / 70% R1234zeZ
                   5  9 0.5 2   % 50% Propylene / 50% Butane
                   6 10 0.8 2   % 80% Propena / 20% IsoPentane
                   7 10 0.5 2   % 50% DME / 50% IsoPentane
                                  ];
                                          
n_results = length(best_solutions(:,1));                     
                     
summary_cell_fixPinch = cell(n_results, 12);   
summary_cell_fixArea = cell(n_results, 12);  
summary_cell_Opt = cell(n_results, 12);  
% --------------------------------------------------------------------- %%
%                    Define  Cycle Parameter                             %
% ---------------------------------------------------------------------  %   

% Set Pressure drops for refrigerant flow in flow direction
    % Evaporator
    cycleInput.dp_evap  = 0*ones(componentInput.HX.n_states_evap-1,1);%  zeros(componentInput.HX.n_states_evap-1,1);
    % Superheater
    cycleInput.dp_sh    = 0*ones(componentInput.HX.n_states_sh-1,1);
    
    % Desuperheater
    cycleInput.dp_desup = 0*ones(componentInput.HX.n_states_desup-1,1);
    % Condenser
    cycleInput.dp_cond  = 0*ones(componentInput.HX.n_states_cond-1,1);   
    % Subcooler
    cycleInput.dp_sub   = 0*ones(componentInput.HX.n_states_sub-1,1);      

% Compressor
    % Constant isentropic compressor efficiency
    cycleInput.Define_Efficiency = 'constant';    % -> componentInput.Comp.eta_is_comp_design is used
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --------------------------------------------------------------------- %%
%                        Fixed Pinch Point                               %
% ---------------------------------------------------------------------  %   
% Heat Exchanger
    cycleInput.DefineHX     = 'Pinch';       % Design approach to set the exact pinch point temperatures in all heat exchangers (Note: E.g. NPV optimization might yield a higher pinch temperature diff.)
    componentInput.HX.delta_T_pinch_source_min = 3;   % [K]
    componentInput.HX.delta_T_pinch_sink_min = 3;     % [K]
    componentInput.HX.delta_T_pinch_sub_min = 3;      % [K]      
    
    
% Optimized Variable
   cycleInput.Objective    = 'COP';            % Optimization yields the highest obtainable coefficient of performance (COP) 

                                                
% --------------------------------------------------------------------- %%
%                        Calculation Options                             %
% ---------------------------------------------------------------------  %      
    economyInput.calcInv    = 'on';     % Define if investment cost should be calculated -> can be replaced by 'off'
    economyInput.calcEco    = 'on';     % Define if economy should be calculated -> can be replaced by 'off'
        
    cycleInput.calcExergy   = 'on';     % Define if exergy analysis is conducted -> can be replaced by 'off'

% --------------------------------------------------------------------- %%
%                           Start Screening                              %
% ---------------------------------------------------------------------  %   

for iter_results=1:n_results
    % Superheating
    delta_T_minSH_vector = [0.05 5];
    cycleInput.delta_T_minSH = delta_T_minSH_vector(best_solutions(iter_results, 4));
    
    % Define Medium
    % Define composition between 0 ... 1
    x_1 = best_solutions(iter_results, 3);
    x_2=1-x_1;
    
    medium_ID1 = best_solutions(iter_results, 1);
    medium_ID2 = best_solutions(iter_results, 2);
    
    % Definition of medium string
    if x_1==1
        medium = {medium_vector{medium_ID1}};
        composition = 1;
    elseif x_1==0
        medium = {medium_vector{medium_ID2}};
        composition = 1;
    else
        medium = {medium_vector{medium_ID1}, medium_vector{medium_ID2}};
        composition = [x_1; x_2];
    end
    
    cycleInput.medium = medium;
    cycleInput.composition = composition;
    
    if x_1==1
        cycleInput.mediumString = strcat(medium_vector_full(medium_ID1));
        simpleMediumString = strcat(medium_vector_full{medium_ID1});
    elseif x_1==0
        cycleInput.mediumString = strcat(medium_vector_full(medium_ID2));
        simpleMediumString = strcat(medium_vector_full{medium_ID2});
    else
        cycleInput.mediumString = strcat(num2str(x_1*100), {' % '},...
            medium_vector_full(medium_ID1), {' / '}, num2str((1-x_1)*100), {' % '}, medium_vector_full(medium_ID2));
        simpleMediumString = strcat(num2str(x_1*100),...
            medium_vector_full{medium_ID1}, '_', num2str((1-x_1)*100), medium_vector_full{medium_ID2});
    end
    
    % Function Call
    try
        clear summary
        % Fluid was not yet simulated -> Run Simulation
        t_opt = tic;
        [COP, summary, cycle] = Opt_HP_std_cycle(boundaryCond, cycleInput, componentInput, solverInput, economyInput);
        toc(t_opt);
        
        summary_cell_fixPinch(iter_results, :) = {cycleInput.mediumString summary.Performance.COP summary.Performance.p_evap/100 summary.Performance.p_cond/100 ...
            summary.Investment.TCI_spec summary.Economy.NPV_total summary.Economy.PBT summary.Economy.c_hg...
            summary.TQ.delta_T_pinch_source summary.TQ.delta_T_pinch_sink summary.Investment.PEC_Evap summary.Investment.PEC_Cond};


    catch
        summary_cell_fixPinch(iter_results, :) = {cycleInput.mediumString NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN};        
    end
    
    
    
end

%% Save Results to Table
table_summary_results_fixPinch = cell2table(summary_cell_fixPinch, 'VariableNames', {'Medium', 'COP', 'p_evap', 'p_cond', 'TCI_spec', 'NPV', 'PBT', 'c_hg',...
    'deltaT_pinch_Source', 'deltaT_pinch_sink', 'PEC_Evap', 'PEC_Cond'})
save('CaseStudy_I/Results/table_summary_BestCasesAnalysis_fixPinch', 'table_summary_results_fixPinch');
save('CaseStudy_I/Results/cell_summary_BestCasesAnalysis_fixPinch', 'summary_cell_fixPinch');

% Optimized Variable
   cycleInput.Objective    = 'NPV';            % Optimization yields the highest obtainable coefficient of performance (COP) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --------------------------------------------------------------------- %%
%                        Fixed HX Area                               %
% ---------------------------------------------------------------------  %   
for iter_results=1:n_results
    % Superheating
    delta_T_minSH_vector = [0.05 5];
    cycleInput.delta_T_minSH = delta_T_minSH_vector(best_solutions(iter_results, 4));
    
    % Define Medium
    % Define composition between 0 ... 1
    x_1 = best_solutions(iter_results, 3);
    x_2=1-x_1;
    
    medium_ID1 = best_solutions(iter_results, 1);
    medium_ID2 = best_solutions(iter_results, 2);
    
    % Definition of medium string
    if x_1==1
        medium = {medium_vector{medium_ID1}};
        composition = 1;
    elseif x_1==0
        medium = {medium_vector{medium_ID2}};
        composition = 1;
    else
        medium = {medium_vector{medium_ID1}, medium_vector{medium_ID2}};
        composition = [x_1; x_2];
    end
    
    cycleInput.medium = medium;
    cycleInput.composition = composition;
    
    if x_1==1
        cycleInput.mediumString = strcat(medium_vector_full(medium_ID1));
        simpleMediumString = strcat(medium_vector_full{medium_ID1});
    elseif x_1==0
        cycleInput.mediumString = strcat(medium_vector_full(medium_ID2));
        simpleMediumString = strcat(medium_vector_full{medium_ID2});
    else
        cycleInput.mediumString = strcat(num2str(x_1*100), {' % '},...
            medium_vector_full(medium_ID1), {' / '}, num2str((1-x_1)*100), {' % '}, medium_vector_full(medium_ID2));
        simpleMediumString = strcat(num2str(x_1*100),...
            medium_vector_full{medium_ID1}, '_', num2str((1-x_1)*100), medium_vector_full{medium_ID2});
    end
    
    % Define HX Investment cost to values of first calculation
    if iter_results==2 
        cycleInput.DefineHX     = 'Invest';          % Design approach to fix the total Investment of source and sink HX and the pinch point for subcooling
        economyInput.HX.PEC_HX_total = summary.Investment.PEC_Evap + summary.Investment.PEC_Cond;               % [], Sum of purchased equipment cost for HX
        componentInput.HX.delta_T_pinch_sub_min = 1;        % [K]   
        componentInput.HX.delta_T_pinch_source_min = 1;     % [K]   <- Value is only used for defining the guess values!
        componentInput.HX.delta_T_pinch_sink_min = 1;       % [K]   <- Value is only used for defining the guess values!
    end
        
    % Function Call
    try
        clear summary
        % Fluid was not yet simulated -> Run Simulation
        t_opt = tic;
        [COP, summary, cycle] = Opt_HP_std_cycle(boundaryCond, cycleInput, componentInput, solverInput, economyInput);
        toc(t_opt);
        
        summary_cell_fixArea(iter_results, :) = {cycleInput.mediumString summary.Performance.COP summary.Performance.p_evap/100 summary.Performance.p_cond/100 ...
            summary.Investment.TCI_spec summary.Economy.NPV_total summary.Economy.PBT summary.Economy.c_hg...
            summary.TQ.delta_T_pinch_source summary.TQ.delta_T_pinch_sink summary.Investment.PEC_Evap summary.Investment.PEC_Cond};


    catch
        summary_cell_fixArea(iter_results, :) = {cycleInput.mediumString NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN};        
    end
    
    
    
end

%% Save Results to Table
table_summary_results_fixArea = cell2table(summary_cell_fixArea, 'VariableNames', {'Medium', 'COP', 'p_evap', 'p_cond', 'TCI_spec', 'NPV', 'PBT', 'c_hg',...
    'deltaT_pinch_Source', 'deltaT_pinch_sink', 'PEC_Evap', 'PEC_Cond'})
save('CaseStudy_I/Results/table_summary_BestCasesAnalysis_fixArea', 'table_summary_results_fixArea');
save('CaseStudy_I/Results/cell_summary_BestCasesAnalysis_fixArea', 'summary_cell_fixArea');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --------------------------------------------------------------------- %%
%                        Optimized HX Area                               %
% ---------------------------------------------------------------------  %   
% Heat Exchanger
    cycleInput.DefineHX     = 'minPinch';       % Design approach to set the minimum pinch point temperatures in all heat exchangers (Note: E.g. NPV optimization might yield a higher pinch temperature diff.)
    componentInput.HX.delta_T_pinch_source_min = 1;   % [K]
    componentInput.HX.delta_T_pinch_sink_min = 1;     % [K]
    componentInput.HX.delta_T_pinch_sub_min = 1;      % [K]   
    
    
% Optimized Variable
   cycleInput.Objective    = 'NPV';            % Optimization yields the highest obtainable coefficient of performance (COP) 

                                                
% --------------------------------------------------------------------- %%
%                        Calculation Options                             %
% ---------------------------------------------------------------------  %      
    economyInput.calcInv    = 'on';     % Define if investment cost should be calculated -> can be replaced by 'off'
    economyInput.calcEco    = 'on';     % Define if economy should be calculated -> can be replaced by 'off'
        
    cycleInput.calcExergy   = 'on';     % Define if exergy analysis is conducted -> can be replaced by 'off'

% --------------------------------------------------------------------- %%
%                           Start Screening                              %
% ---------------------------------------------------------------------  %   

for iter_results=1:n_results
    % Superheating
    delta_T_minSH_vector = [0.05 5];
    cycleInput.delta_T_minSH = delta_T_minSH_vector(best_solutions(iter_results, 4));
    
    % Define Medium
    % Define composition between 0 ... 1
    x_1 = best_solutions(iter_results, 3);
    x_2=1-x_1;
    
    medium_ID1 = best_solutions(iter_results, 1);
    medium_ID2 = best_solutions(iter_results, 2);
    
    % Definition of medium string
    if x_1==1
        medium = {medium_vector{medium_ID1}};
        composition = 1;
    elseif x_1==0
        medium = {medium_vector{medium_ID2}};
        composition = 1;
    else
        medium = {medium_vector{medium_ID1}, medium_vector{medium_ID2}};
        composition = [x_1; x_2];
    end
    
    cycleInput.medium = medium;
    cycleInput.composition = composition;
    
    if x_1==1
        cycleInput.mediumString = strcat(medium_vector_full(medium_ID1));
        simpleMediumString = strcat(medium_vector_full{medium_ID1});
    elseif x_1==0
        cycleInput.mediumString = strcat(medium_vector_full(medium_ID2));
        simpleMediumString = strcat(medium_vector_full{medium_ID2});
    else
        cycleInput.mediumString = strcat(num2str(x_1*100), {' % '},...
            medium_vector_full(medium_ID1), {' / '}, num2str((1-x_1)*100), {' % '}, medium_vector_full(medium_ID2));
        simpleMediumString = strcat(num2str(x_1*100),...
            medium_vector_full{medium_ID1}, '_', num2str((1-x_1)*100), medium_vector_full{medium_ID2});
    end
    
    % Function Call
    try
        clear summary
        % Fluid was not yet simulated -> Run Simulation
        t_opt = tic;
        [COP, summary, cycle] = Opt_HP_std_cycle(boundaryCond, cycleInput, componentInput, solverInput, economyInput);
        toc(t_opt);
        
        summary_cell_Opt(iter_results, :) = {cycleInput.mediumString summary.Performance.COP summary.Performance.p_evap/100 summary.Performance.p_cond/100 ...
            summary.Investment.TCI_spec summary.Economy.NPV_total summary.Economy.PBT summary.Economy.c_hg...
            summary.TQ.delta_T_pinch_source summary.TQ.delta_T_pinch_sink summary.Investment.PEC_Evap summary.Investment.PEC_Cond};


    catch
        summary_cell_Opt(iter_results, :) = {cycleInput.mediumString NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN};        
    end
    
    
    
end

%% Save Results to Table
table_summary_results_Opt = cell2table(summary_cell_Opt, 'VariableNames', {'Medium', 'COP', 'p_evap', 'p_cond', 'TCI_spec', 'NPV', 'PBT', 'c_hg',...
    'deltaT_pinch_Source', 'deltaT_pinch_sink', 'PEC_Evap', 'PEC_Cond'})
save('CaseStudy_I/Results/table_summary_BestCasesAnalysis_Opt', 'table_summary_results_Opt');
save('CaseStudy_I/Results/cell_summary_BestCasesAnalysis_Opt', 'summary_cell_Opt');

%% -------------------------------------------------------------------- %%
%                        Plot Figures                                    %
% ---------------------------------------------------------------------  %  
% Figure inputs
    font = 'Times New Roman';
    fs = 10;
    show_legend=0;
% Dimensions

    width_subplot = 6;        % cm
    height_subplot = 2.8;         % cm

    space_bottom = 3;
    if show_legend==0
        height_legend=0;
    else
        height_legend = 0.5;% 1.2;%1.2;% 1.2;
    end
    width_legend = 3; % 3
    space_vertical = 0.2;
    space_top = height_legend+3*space_vertical;% 2.5;
    
    space_left = 1.2;
    space_right = 0.3;
    
    figure_width = space_left + width_subplot + space_right;     % cm
    figure_height = space_bottom + 2*height_subplot + space_top;  % cm

COP_comparison = [cell2mat(summary_cell_fixPinch(:,2)) cell2mat(summary_cell_fixArea(:,2)) cell2mat(summary_cell_Opt(:,2))]; 
NPV_comparison = [cell2mat(summary_cell_fixPinch(:,6)) cell2mat(summary_cell_fixArea(:,6)) cell2mat(summary_cell_Opt(:,6))]; 
PBT_comparison = [cell2mat(summary_cell_fixPinch(:,7)) cell2mat(summary_cell_fixArea(:,7)) cell2mat(summary_cell_Opt(:,7))]; 
PEC_HXsource_comparison = [cell2mat(summary_cell_fixPinch(:,11)) cell2mat(summary_cell_fixArea(:,11)) cell2mat(summary_cell_Opt(:,11))]; 
PEC_HXsink_comparison  = [cell2mat(summary_cell_fixPinch(:,12)) cell2mat(summary_cell_fixArea(:,12)) cell2mat(summary_cell_Opt(:,12))]; 

figure_comparison = figure('Name', 'Overview comparison', 'Units','centimeters',...
                   'Position', [10 5 figure_width figure_height], ...
                   'PaperSize', [figure_width figure_height], ...
                   'PaperPosition', [10 5 figure_width figure_height]);
               
ax1=axes('ylim', [1 8], 'Units', 'centimeters', ...
    'Position', [space_left space_bottom+height_subplot+space_vertical width_subplot height_subplot],...
    'xTick', [1 2 3 4 5 6 7 8], 'xTickLabel', {'', '','','','','','',''});
hold on;
grid on;
box on;
bar(COP_comparison)

y1=ylabel('COP, -','Rotation',90, 'FontName', font, 'FontSize', fs);

ax2=axes('Units', 'centimeters', ...
    'Position', [space_left space_bottom width_subplot height_subplot], ...
    'Colormap', gray);
hold on;
grid on;
box on;
bar(NPV_comparison/1000)
% xticklabels(summary_opt(:,1))
% xtickangle(45)
% categorical(summary_opt(:,1))
y1=ylabel('NPV, 1000 ','Rotation',90, 'FontName', font, 'FontSize', fs);
% legend('a) fixed Pinch', 'b) fixed HX Inv.', 'c) NPV optimized', 'Location', 'EastOutside');
set(ax2, 'ylim', [0 800], 'xTick', [1 2 3 4 5 6 7 8], 'XTickLabel', {cell2mat(summary_cell_fixPinch{1,1}) cell2mat(summary_cell_fixPinch{2,1}) ...
    cell2mat(summary_cell_fixPinch{3,1}) cell2mat(summary_cell_fixPinch{4,1}) ...
    cell2mat(summary_cell_fixPinch{5,1}) cell2mat(summary_cell_fixPinch{6,1}) ...
    cell2mat(summary_cell_fixPinch{7,1}) cell2mat(summary_cell_fixPinch{8,1})} ,...
    'xTickLabelRotation', 30,...
    'FontName', font, 'FontSize', fs); % cell2mat(summary_cell_fixPinch{8,1}) ,...
    % 'TickLabelInterpreter', 'Latex'

%         colormap(ax1, 'flat');
if show_legend==1    
legend_complete = legend('a) fixed Pinch', 'b) fixed HX Inv.', 'c) NPV optimized',...
    'Orientation', 'Vertical'); % ,   'Position', [space_left space_bottom+2*space_vertical+2*height_subplot width_subplot height_legend]

set(legend_complete,'Units', 'Centimeters',...
    'Position',[space_left+0.5*(width_subplot-width_legend) space_bottom+2*space_vertical+2*height_subplot width_legend height_legend]);
end
% title('n=20 years'); 'Location', 'NorthOutside'   'Orientation', 'Vertical', 
%     'Units', 'centimeters',...
%     'Position', [10 1 width_subplot 1]

w_tb=2;
h_tb=0.5;

% ax1.Position
annotation(figure_comparison,'textbox',...
     'Units', 'Centimeters',...
     'Position', [(ax1.Position(1)+ax1.Position(3))-w_tb*1.05 (ax1.Position(2)+ax1.Position(4))-(h_tb*1.2) w_tb h_tb],...
    'String','Case I',...
    'VerticalAlignment', 'middle', 'HorizontalAlignment','center',...
    'FontName', font, 'FontSize', fs,...
    'FitBoxToText','off', 'BackgroundColor',[1 1 1]);  

% cell2mat(summary_cell_fixPinch(:,1))
% summary_cell_fixPinch{1,1} summary_cell_fixPinch(2,1) cell2mat(summary_cell_fixPinch{3,1}) ...
%    summary_cell_fixPinch{4,1} summary_cell_fixPinch{5,1} summary_cell_fixPinch{6,1} summary_cell_fixPinch{7,1}
% {'Ammonia' 'Butane' 'R-1234yf' 'x' 'x' 'x' 'x'}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Full plot
figure_comparison = figure('Name', 'Overview comparison');
subplot(4,1,1, 'ylim', [1 9])
hold on;
grid on;
box on;
bar(COP_comparison)
% xticklabels(summary_opt(:,1))
% xtickangle(45)
% categorical(summary_opt(:,1))
y1=ylabel('COP, -','Rotation',90, 'FontName', font, 'FontSize', fs);
legend('fixed Pinch', 'fixed HX Inv.', 'NPV optimized', 'Location', 'EastOutside');
% title('n=20 years');

subplot(4,1,2)
hold on;
grid on;
box on;
bar(NPV_comparison)
% xticklabels(summary_opt(:,1))
% xtickangle(45)
% categorical(summary_opt(:,1))
y1=ylabel('NPV, ','Rotation',90, 'FontName', font, 'FontSize', fs);
legend('fixed Pinch', 'fixed HX Inv.', 'NPV optimized', 'Location', 'EastOutside');

subplot(4,1,3)
hold on;
grid on;
box on;
bar(PBT_comparison)
% bar(HXarea_source_comparison+HXarea_sink_comparison)
% xticklabels(summary_opt(:,1))
% xtickangle(45)
% categorical(summary_opt(:,1))
y1=ylabel('PBT, years','Rotation',90, 'FontName', font, 'FontSize', fs);
legend('fixed Pinch', 'fixed HX Inv.', 'NPV optimized', 'Location', 'EastOutside');

subplot(4,1,4)
hold on;
grid on;
box on;
bar(PEC_HXsource_comparison+PEC_HXsink_comparison)
% xticklabels(summary_opt(:,1))
% xtickangle(45)
% categorical(summary_opt(:,1))
y1=ylabel('HX Area total, m2','Rotation',90, 'FontName', font, 'FontSize', fs);
legend('fixed Pinch', 'fixed HX Inv.', 'NPV optimized', 'Location', 'EastOutside');
