%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
% -------------------------------------------------------------------------

% --------------------------------------------------------------------- %%
%                          Load Inputs                                   %
% ---------------------------------------------------------------------  %   
load('CaseStudy_I/Inputs_CaseStudy_I.mat');

% Load list of fluids
ListOfFluids;

% Define folder path into which the results are saved
result_path = 'CaseStudy_I/Results/20180719/';
mkdir(result_path);

% Check Refprop Version
% display(strcat({'Refprop Version: '}, {num2str(refpropm_v1('0','c', 0,'',0, {'water'}, 1))}))  ; 

% --------------------------------------------------------------------- %%
%                    Define  Cycle Parameter                             %
% ---------------------------------------------------------------------  %   
    % Superheating
    SH_index=2;
    delta_T_minSH_vector = [0.05 5];
    cycleInput.delta_T_minSH = delta_T_minSH_vector(SH_index);

% Set Pressure drops for refrigerant flow in flow direction
    % Evaporator
    cycleInput.dp_evap  = 0*ones(componentInput.HX.n_states_evap-1,1);%  zeros(componentInput.HX.n_states_evap-1,1);
    % Superheater
    cycleInput.dp_sh    = 0*ones(componentInput.HX.n_states_sh-1,1);
    
    % Desuperheater
    cycleInput.dp_desup = 0*ones(componentInput.HX.n_states_desup-1,1);
    % Condenser
    cycleInput.dp_cond  = 0*ones(componentInput.HX.n_states_cond-1,1);   
    % Subcooler
    cycleInput.dp_sub   = 0*ones(componentInput.HX.n_states_sub-1,1);      

% Compressor
    % Constant isentropic compressor efficiency
    cycleInput.Define_Efficiency = 'constant';    % -> componentInput.Comp.eta_is_comp_design is used
    
% --------------------------------------------------------------------- %%
%                  Define  Boundary Conditions                           %
% ---------------------------------------------------------------------  %   
% Heat Exchanger
    cycleInput.DefineHX     = 'Pinch';       % Design approach to set the exact pinch point temperatures in all heat exchangers (Note: E.g. NPV optimization might yield a higher pinch temperature diff.)
    componentInput.HX.delta_T_pinch_source_min = 3;   % [K]
    componentInput.HX.delta_T_pinch_sink_min = 3;     % [K]
    componentInput.HX.delta_T_pinch_sub_min = 3;      % [K]      
    
    
% Optimized Variable
   cycleInput.Objective    = 'COP';            % Optimization yields the highest obtainable coefficient of performance (COP) 

                                                
% --------------------------------------------------------------------- %%
%                        Calculation Options                             %
% ---------------------------------------------------------------------  %      
    economyInput.calcInv    = 'on';     % Define if investment cost should be calculated -> can be replaced by 'off'
    economyInput.calcEco    = 'on';     % Define if economy should be calculated -> can be replaced by 'off'
        
    cycleInput.calcExergy   = 'on';     % Define if exergy analysis is conducted -> can be replaced by 'off'

% --------------------------------------------------------------------- %%
%                           Start Screening                              %
% ---------------------------------------------------------------------  %   

for medium_ID1=1:n_media
    for medium_ID2=(medium_ID1+1):n_media
        for i_x=1:n_x
            % Define Medium
            % Define composition between 0 ... 1
            x_1=x_1_vector(i_x);
            x_2=1-x_1;
            
            % Definition of medium string
            if x_1==1
                medium = {medium_vector{medium_ID1}};
                composition = 1;
            elseif x_1==0
                medium = {medium_vector{medium_ID2}};
                composition = 1;
            else
                medium = {medium_vector{medium_ID1}, medium_vector{medium_ID2}};
                composition = [x_1; x_2];
            end
            
            cycleInput.medium = medium;
            cycleInput.composition = composition;
            
            if x_1==1
                cycleInput.mediumString = strcat(medium_vector_full(medium_ID1));
                simpleMediumString = strcat(medium_vector_full{medium_ID1});
            elseif x_1==0
                cycleInput.mediumString = strcat(medium_vector_full(medium_ID2));
                simpleMediumString = strcat(medium_vector_full{medium_ID2});
            else
                cycleInput.mediumString = strcat(num2str(x_1*100), {' % '},...
                    medium_vector_full(medium_ID1), {' / '}, num2str((1-x_1)*100), {' % '}, medium_vector_full(medium_ID2));
                simpleMediumString = strcat(num2str(x_1*100),...
                    medium_vector_full{medium_ID1}, '_', num2str((1-x_1)*100), medium_vector_full{medium_ID2});
            end
            
            % Function call

                % Check if fluid was already simulated
                if exist(strcat(result_path, simpleMediumString, '.mat'))==0
                    try
                        % Fluid was not yet simulated -> Run Simulation
                        t_opt = tic;
                        cycleInput.DefineHX     = 'Pinch'; 
                        [COP, summary, cycle] = Opt_HP_std_cycle(boundaryCond, cycleInput, componentInput, solverInput, economyInput);
                        toc(t_opt);
                        
                        if summary.exitflag==-2
                            % If no feasible solution was found, equality
                            % constraints are defined as inequality
                            % constraints and simulation is repeated
                            t_opt = tic;
                            cycleInput.DefineHX     = 'minPinch'; 
                            [COP, summary, cycle] = Opt_HP_std_cycle(boundaryCond, cycleInput, componentInput, solverInput, economyInput);
                            display(strcat('Medium: ',{' '}, num2str(x_1*100), ' %', {' '}, medium_vector{medium_ID1}, {' -'},{' '}, num2str(x_2*100), ' %', {' '}, medium_vector{medium_ID2}, {': no feasible solution was found, simulation is repeated with released constraints'}))
                            toc(t_opt);                          
                        end           
                        
                        % Save result into result folder
                        save(strcat(result_path, simpleMediumString, '.mat'), 'summary');
                        
                        display(strcat('Medium: ',{' '}, num2str(x_1*100), ' %', {' '}, medium_vector{medium_ID1}, {' -'},{' '}, num2str(x_2*100), ' %', {' '}, medium_vector{medium_ID2}, {': COP = '}, num2str(COP)))
                    catch
                        clear summary
                        summary.mediumString = cycleInput.mediumString;
                        summary.Performance.COP = NaN;
                        
                        % Save result into result folder
                        save(strcat(result_path, simpleMediumString, '.mat'), 'summary');   
                        
                        display(strcat('Medium: ',{' '}, num2str(x_1*100), ' %', {' '}, medium_vector{medium_ID1}, {' -'},{' '}, num2str(x_2*100), ' %', {' '}, medium_vector{medium_ID2}, {': Fluid could not be solved'}))
                    end

                else
                    % Fluid was already simulated
                    display(strcat('Medium: ',{' '}, num2str(x_1*100), ' %', {' '}, medium_vector{medium_ID1}, {' -'},{' '}, num2str(x_2*100), ' %', {' '}, medium_vector{medium_ID2}, {': Fluid already evaluated'}))
                end
            
        end
    end
end


