% This file defines the inputs for the analysis of case study I: 
%   "Excess heat recovery from data center cooling for district heating supply"

%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
%% ------------------------------------------------------------------------

% close all;
clear all;
clc;

addpath(genpath('CaseStudy_II'));
addpath(genpath('HPOptimizationCalls'));
addpath(genpath('HPCycles'));
addpath(genpath('HPEconomics'));
addpath(genpath('Figures'));
addpath(genpath('Miscellaneous'));
addpath(genpath('C:\Program Files\REFPROP'));   % This is where the refpropm.m should be placed according to Refprop
addpath(genpath('C:\Program Files\Coolprop'));  % Here are the functions for Coolprop placed

%% Input Parameter - Boundary Conditions
%  All Inlet and Outlet Temperatures of Sink and Source are known
%  Heat load of heat sink is known
%  Mass flow rates are unknown<z


    % Heat Source
        % Define parameter to define the heat source: boundaryCond.DefineSource   = ...
        % - Inlet and outlet Temperature: 'T_in + T_out' -> supply boundaryCond.T_source_in and boundaryCond.T_source_out; (in [K]) 
        % - Inlet temp. and m_flow: 'T_in + m_flow'      -> supply boundaryCond.T_source_in and boundaryCond.m_flow_source; (in [kg/s])

    boundaryCond.DefineSource = 'T_in + T_out';
    boundaryCond.T_source_in = 273.15+40;           % [K]
    boundaryCond.T_source_out = 273.15+25;          % [K]
    boundaryCond.p_source = 1e5;                    % [Pa]
    boundaryCond.medium_source = 'water';
   
    % Heat Sink
    boundaryCond.T_sink_in = 273.15+50;             % [K]
    boundaryCond.T_sink_out = 273.15+75;            % [K]
    boundaryCond.p_sink = 5e5;                      % [Pa]
    boundaryCond.medium_sink = 'water';

    % Load
        % Define parameter to define the dimensioning of the cycle among: cycleInput.DefineLoad   = ...
        % - Supplied heat load:     'Q_dot_sink'        -> supply cycleInput.Q_dot_sink = 300000; (in [W])
        % - Supplied cooling load:  'Q_dot_source'      -> supply cycleInput.Q_dot_source = 300000; (in [W]) 
        % - Compressor inlet V_dot: 'V_dot_comp_in'     -> supply cycleInput.V_dot_comp_in = 0.01; (in [m3/s])

%     cycleInput.DefineLoad   = 'Q_dot_sink';     % Fixed supplied heat load
%     cycleInput.Q_dot_sink = 100000;             % [W]
%     
    cycleInput.DefineLoad   = 'Q_dot_source';  % Fixed supplied cooling load
    cycleInput.Q_dot_source = 500000;             % [W]
    
%     cycleInput.DefineLoad    = 'V_dot_comp_in';  % Fixed volume flow rate at compressor inlet
%     cycleInput.V_dot_comp_in = 300/3600;        % [m3/s]

%     cycleInput.DefineLoad    = 'V_dot_comp_disp';  % Fixed displacement rate at compressor inlet
%     cycleInput.V_dot_comp_disp = 300/3600;        % [m3/s]  
    
    % Dead state for exergy analysis
    boundaryCond.T_amb = 273.15 + 10;               % [K]  Ambient temperature
    boundaryCond.p_amb = 1.0e5;                     % [Pa] Ambient pressure    
    
%% Heat Pump Cycle Parameter
  
% Heat Exchangers ---------------------------------------------------------
    % Temperature Pinch Differences
    componentInput.HX.delta_T_pinch_source_min = 3;   % [K]
    componentInput.HX.delta_T_pinch_sink_min = 3;     % [K]
    componentInput.HX.delta_T_pinch_ihx_min = 3;      % [K]
    
    % Discretization 
    componentInput.HX.n_states_evap = 20;             % 50 for UA value iteration
    componentInput.HX.n_states_sh = 4;                %  4 for UA value iteration
    componentInput.HX.n_states_cond = 20;             % 25 for UA value iteration
    componentInput.HX.n_states_desup = 5;             %  5 for UA value iteration
    componentInput.HX.n_states_sub = 5;               %  5 for UA value iteration
    componentInput.HX.n_states_ihx = 5;               %  5 for UA value iteration    

% Compressor --------------------------------------------------------------
    % Efficiencies
    componentInput.Comp.eta_is_comp_design = 0.75;    % [-] Isentropic Efficiency
    componentInput.Comp.heatloss = 0.00;              % [-] 0.0 ... 1.0 - Heat loss of motor as share of difference between adiabatic and isentropic expansion
    componentInput.Comp.eta_vol_comp = 0.9;           % [-] Volumetric Efficiency
    componentInput.Comp.eta_motor = 0.95;             % [-] Motor Efficiency
    % Discretization
    componentInput.Comp.n_states_comp = 2;

% %% Numerical input parameter
    solverInput.limit_sc        = 0.95;             % Limit above which process is considered as supercritcal
    solverInput.tol_delta_T     = 1e-3; %-4           % Allowable error on pinch temperature    
%     iter_max = 50;                      % Number of maximum iterations
    solverInput.t_max = 30;                         % Maximum time for iteration in [s]

% Solver ------------------------------------------------------------------
% Show Debugging information
    solverInput.show_info=0;              % If 1, information during solving process is shown

%% Economics
% Investment
    economyInput.f_TCI = 4.0;                  % [-] Factor for estimation of TCI: TCI=f_TCI*PEC
    economyInput.f_OM = 0.0;                     % [-] Factor for estimation of O&M cost as one time investment: O&M=TCI*f_OM
    
% Interest Rates
%     economyInput.interest = 0.07;               % [-] Interest Rate 
%     economyInput.inflation = 0.02;              % [-] Inflation Rate
    economyInput.i_eff = 0.05;% (1+economyInput.interest)/(1+economyInput.inflation)-1;       % [-] Effective Interest Rate 
    
% Operation
    economyInput.n_lt = 20;                     % [years] Lifetime
    economyInput.OH = 8000;                     % [h/year] Operation Hours
    
% Fuel Cost
    economyInput.c_el = 0.120;                  % [/kWh] Specific cost of electricity
    economyInput.c_hsource = 0;                 % [/kWh] Specific cost of heat from heat source
    economyInput.c_alt = 0.04;                  % [/kWh] Specific cost of heat from alternative heat supply    
   
% Alternative heat supply 
    economyInput.eta_alt = 1;                 % [-] Efficiency of alternative heat supply (e.g. boiler efficiency) between 0...1
       
%% --------------------------------------------------------------------- %%
%                                Save Inputs                              %
%  ---------------------------------------------------------------------  %   
save('CaseStudy_II/Inputs_CaseStudy_II.mat');