function [figure_logph] = Function_logph_diagram(summary,cycle,options)
%% Script provides a log(p)-h-diagram for the calculated state points

%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
% -------------------------------------------------------------------------

% Medium
medium = cycle.medium;
composition = cycle.composition;

% Get variables from summary result structures
h_cycle = cycle.h_complete;
p_cycle = cycle.p_complete;

%% Calculate critical properties
    % Call SATSPLN
    Run_SATSPLN=1;    % if = 1, SATSPLN is called
    save('C:\Program Files\REFPROP\Run_SATSPLN', 'Run_SATSPLN');
    [T_crit, p_crit, h_crit, s_crit]  = refpropm_v1('Tphs', 'c', 0,'',0, medium, composition);
    % p_crit
    Run_SATSPLN=0;
    save('C:\Program Files\REFPROP\Run_SATSPLN', 'Run_SATSPLN');

%% Defining Inputs
%  Defining which lines to plot
    plot_isothermals = 1;       % If = 1, then Isothermals are plotted
    plot_isentropes = 1;        % If = 1, then Isentropes are plotted
    plot_isoquality = 1;        % If = 1, then Lines of equal quality are plotted

% Number of isothermals and isentropes
    % Number of isothermals
    n_T = options.n_T;
    % Number of isentropes above and below critical point
    n_s = options.n_s;
    
% Resolution of isothermals and isentropes    
    % Number of calculated points along the displayed pressure (for isothermals) in liquid, 2phase and gas   
    n_p_isothermal_liq      = 10;
    n_p_isothermal_2phase   = 10;    
    n_p_isothermal_gas      = 40;  

    n_p_isothermal = n_p_isothermal_liq + n_p_isothermal_2phase + n_p_isothermal_gas + 2; % 100   
    
    % Number of calculated points along the displayed pressure (for isentropes)   
    n_p_isentropes = 10;
    
%% Defining Displayed Range of Pressure
    p_display = [min(0.5*min(p_cycle), 1000) 1.5*max(p_crit, max(p_cycle))] ; 
    
%% Calculating the Vapor Dome 
    %  Boundaries for pressure
    limit_high = 0.90;      % in % of critical pressure
    limit_low  = 0.05;      % in % of critical pressure

    % Number of calculated points on the saturation line
    n_sat = 50;
    % Number of points for discretization of critical region
    n_h_critical = 10;

    % Preallocating space in advance
    h_bubble = zeros(n_sat,1);
    h_dew = zeros(n_sat,1);

    p_sat = linspace(p_display(1,1), limit_high*p_crit, n_sat); % limit_low*p_crit
    p_sat_total = linspace(p_display(1,1), p_crit, n_sat); % limit_low*p_crit

    % calculate dew line properties up to limit below critical point
    for i=1:n_sat
       h_bubble(i) = refpropm_v1('h', 'p', p_sat(i), 'Q', 0, medium, composition);
       h_dew(i)    = refpropm_v1('h', 'p', p_sat(i), 'Q', 1, medium, composition); 
    end
    
    % Create Spline for critical region of vapor dome
    h_sat_crit = linspace(h_bubble(n_sat), h_dew(n_sat), n_h_critical);
    
    p_sat_crit_spline = spline([h_bubble(n_sat-1) h_bubble(n_sat) h_crit h_dew(n_sat)], ...
                                   [p_sat(n_sat-1) p_sat(n_sat) p_crit p_sat(n_sat)], ...
                                   h_sat_crit);
% This version includes one more point on the dew line
% -> If one more point is included, the shape of the vapor dome is more
% "smooth", but yields a slightly wrong critical point
%     p_sat_crit_spline = spline([h_bubble(n_sat-1) h_bubble(n_sat) h_crit h_dew(n_sat) h_dew(n_sat-1)], ...
%                                    [p_sat(n_sat-1) p_sat(n_sat) p_crit p_sat(n_sat) p_sat(n_sat-1)], ...
%                                    h_sat_crit);

%% Fixing the range of displayed specific enthalpy
    
    h_display = [min(h_bubble)-5e4 max([h_dew' h_cycle])+5e4];

    % discretize enthalpy in displaying space
    % h_display_disc = linspace(h_display(1,1)-1e5, h_display(1,2)+1e5, n_h_isothermal); % 
    % h_display_disc = linspace(7e5, 8e5, n_h_isothermal); % -1e5
    
%% Calculate Isothermals and Isentropes
    
try
    s_isentrope_min = refpropm_v1('s', 'p', p_display(1), 'h', h_display(1), medium, composition);
catch
    s_isentrope_min = refpropm_v1('s', 'p', p_sat(1), 'h', h_bubble(1), medium, composition);
end
s_isentrope_max = refpropm_v1('s', 'p', p_display(1), 'h', h_display(2), medium, composition);
delta_s_isentrope = round((s_isentrope_max - s_isentrope_min) / n_s / 100) * 100;

n_s = round((s_isentrope_max-s_isentrope_min)/delta_s_isentrope) + 1;

% T_isothermal_min = refpropm_v1('T', 'p', p_display(1), 'h', h_display(1), medium, composition);
T_isothermal_min = refpropm_v1('T', 'p', p_display(1), 's', s_isentrope_min, medium, composition);
T_isothermal_max = refpropm_v1('T', 'p', p_display(2), 'h', h_display(2), medium, composition);
delta_T_isothermal = max(round((T_isothermal_max - T_isothermal_min) / n_T / 20) * 20, 20);

n_T = round((T_isothermal_max-T_isothermal_min)/delta_T_isothermal) + 1;



    if strcmp(options.isothermals, 'on')==1 % plot_isothermals == 1
%         T_isothermal = linspace(round(T_crit/delta_T_isothermal)*delta_T_isothermal - n_T*delta_T_isothermal, ...
%                                 round(T_crit/delta_T_isothermal)*delta_T_isothermal + n_T*delta_T_isothermal, ... 
%                                 n_T*2+1);
        T_isothermal = linspace(round((T_isothermal_min-273.15)/delta_T_isothermal) * delta_T_isothermal, ...
                                round((T_isothermal_min-273.15)/delta_T_isothermal) * delta_T_isothermal + n_T*delta_T_isothermal, ... 
                                n_T+1)+273.15;
    end                    
    
    if strcmp(options.isentropes, 'on')==1 
%         s_isentrope = linspace(round(s_crit/delta_s_isentrope)*delta_s_isentrope - n_s*delta_s_isentrope, ...
%                                 round(s_crit/delta_s_isentrope)*delta_s_isentrope + n_s*delta_s_isentrope, ... 
%                                 n_s*2+1);    
        s_isentrope = linspace(round(s_isentrope_min/delta_s_isentrope)*delta_s_isentrope, ...
                                round(s_isentrope_min/delta_s_isentrope)*delta_s_isentrope + n_s*delta_s_isentrope, ... 
                                n_s+1);       
    end
    
    % calculate pressure for the pairs of T and h
    h_isothermals = NaN(n_p_isothermal, (n_T)); % Preallocate space
    p_isothermals = NaN(n_p_isothermal, (n_T)); % Preallocate space
    
    h_isentropes = NaN(n_p_isentropes, (n_s)); % Preallocate space 2*n_s+1
    p_isentropes = NaN(n_p_isentropes, (n_s)); % Preallocate space 2*n_s+1    
    
    % Logarithmically constant discretized pressure range for calculation of isothermals
    p_display_disc = (logspace(log10(p_display(1,1)), log10(p_display(1,2)), n_p_isothermal));
    
    % Equally discretized pressure range for calculation of isothermals
    % p_display_disc = linspace(p_display(1,1), p_display(1,2), n_p_isothermal);    
    
    % Isothermals   
    if strcmp(options.isothermals, 'on')==1 % plot_isothermals == 1
        for i=1:(n_T)
            
            % -> Check in which phase
            % -> Discretize for gas/two-phase/liquid
            % -> Introduce new matrix for p_isothermals
            
            Q = refpropm_v1('Q', 'T', T_isothermal(i), 'p', p_display_disc(1), medium, composition);
            Q_vector(i)=Q;
            %             for ii=1:n_p_isothermal
            
            
            %             p_bubble    = refpropm_v1('p', 'T', T_isothermal(i), 'Q', 0, medium, composition);
            %             p_dew       = refpropm_v1('p', 'T', T_isothermal(i), 'Q', 1, medium, composition);
            
            
            if Q<=0
                % Isothermal line completely in liquid phase
                
                p_isothermals(1:n_p_isothermal_liq,i) = (logspace(log10(p_display(1,1)), log10(p_display(1,2)), n_p_isothermal_liq));
                for ii=1:n_p_isothermal_liq
                    try
                        h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
                    catch
                        h_isothermals(ii,i) = NaN;
                    end
                    
                end
            elseif (Q>0) && (Q<=1)
                % Isothermal line starts in two phase zone
                
                % 2phase zone -> Discretization in equal quality
                Q_2phase = linspace(Q, 0, n_p_isothermal_2phase);
                for iii=1:n_p_isothermal_2phase
                    try
                        p_isothermals(iii,i) = refpropm_v1('p', 'T', T_isothermal(i), 'Q', Q_2phase(iii), medium, composition);
                    catch
                        p_isothermals(iii,i) = NaN;
                    end
                    
                    try
                    % refpropm_v1('p', 'T', T_isothermal(i), 'Q', Q_2phase(iii), medium, composition)
                        h_isothermals(iii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'Q', Q_2phase(iii), medium, composition);
                    catch
                        h_isothermals(iii,i) = NaN;
                    end
                end
                % Liquid zone
                p_isothermals(n_p_isothermal_2phase+1:n_p_isothermal_2phase+n_p_isothermal_liq,i) = (logspace(log10(p_isothermals(n_p_isothermal_2phase,i)), log10(p_display(1,2)), n_p_isothermal_liq));
                
                for ii=n_p_isothermal_2phase+1:n_p_isothermal_2phase+n_p_isothermal_liq
                    try
%                         h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
                        % h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
                        
%                         tic;
                        h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
%                         runtime=toc;
%                         display(strcat({'T = '},num2str(T_isothermal(i)-273.15), {' C, p = '}, num2str(p_isothermals(ii,i)/100), ...
%                             {' bar; runtime = '}, num2str(runtime), {' s'}))
                    catch
                        h_isothermals(ii,i) = NaN;
%                         runtime=toc;               
%                         ii
%                         display(strcat({'T = '},num2str(T_isothermal(i)-273.15), {' C, p = '}, num2str(p_isothermals(ii,i)/100), ...
%                             {' bar; DID NOT CONVERGE after: runtime = '}, num2str(runtime), {' s'}))
                    end
                end
            elseif (Q>1) && (T_isothermal(i)<=refpropm_v1('T', 'p', limit_high*p_crit, 'Q', 0, medium, composition))
                % Isothermal line starts in gas phase and is subcritical

                
                % Gas zone 
                p_isothermals(1:n_p_isothermal_gas,i) = (logspace(log10(p_display(1,1)), log10(refpropm_v1('p', 'T', T_isothermal(i), 'Q', 1, medium, composition)), n_p_isothermal_gas));   % p_isothermals(n_p_isothermal_gas+1,i)
                
                for ii=1:n_p_isothermal_gas-1
                    try
%                         h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
                        h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);

                    catch
                        h_isothermals(ii,i) = NaN;
                    end
                end           
                try
                    %                         h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
                    h_isothermals(n_p_isothermal_gas,i) = refpropm_v1('h', 'T', T_isothermal(i), 'Q', 1, medium, composition);
                    
                catch
                    h_isothermals(ii,i) = NaN;
                end
                
                % 2phase zone -> Discretization in equal quality
                Q_2phase = linspace(1, 0, n_p_isothermal_2phase);
                for iii=1:(n_p_isothermal_2phase)
                    try
                        p_isothermals(iii+n_p_isothermal_gas,i) = refpropm_v1('p', 'T', T_isothermal(i), 'Q', Q_2phase(iii), medium, composition);
                    catch
                        p_isothermals(iii+n_p_isothermal_gas,i) = NaN;
                    end
                    
                    try
                    % refpropm_v1('p', 'T', T_isothermal(i), 'Q', Q_2phase(iii), medium, composition)

                        h_isothermals(iii+n_p_isothermal_gas,i) = refpropm_v1('h', 'T', T_isothermal(i), 'Q', Q_2phase(iii), medium, composition);
                    catch
                        h_isothermals(iii+n_p_isothermal_gas,i) = NaN;
                    end
                end
                % Liquid zone
                p_isothermals((n_p_isothermal_gas+n_p_isothermal_2phase+1):(n_p_isothermal_gas+n_p_isothermal_2phase+n_p_isothermal_liq),i) = ...
                    (logspace(log10(p_isothermals(n_p_isothermal_gas+n_p_isothermal_2phase,i)), log10(p_display(1,2)), n_p_isothermal_liq));

                try
                    h_isothermals(n_p_isothermal_gas+n_p_isothermal_2phase+1,i) = refpropm_v1('h', 'T', T_isothermal(i), 'Q', 0, medium, composition);
                catch
                    h_isothermals(n_p_isothermal_gas+n_p_isothermal_2phase+1,i) = NaN;
                end
                for ii=(n_p_isothermal_gas+n_p_isothermal_2phase+2):(n_p_isothermal_gas+n_p_isothermal_2phase+n_p_isothermal_liq)
                    try
                        h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
                    catch
                        h_isothermals(ii,i) = NaN;
                    end
                end
                
            elseif (Q>1) && (T_isothermal(i)>T_crit*limit_high)
                % Isothermal line starts in gas phase and is supercritical
                 p_isothermals(1:n_p_isothermal,i) = (logspace(log10(p_display(1,1)), log10(p_display(1,2)), n_p_isothermal));
                
                for ii=1:n_p_isothermal
                    try
                        h_isothermals(ii,i) = refpropm_v1('h', 'T', T_isothermal(i), 'p', p_isothermals(ii,i), medium, composition);
                    catch
                        h_isothermals(ii,i) = NaN;
                    end
                end                    
            end
        end
    end
    
    p_display_disc = (logspace(log10(p_display(1,1)), log10(p_display(1,2)), n_p_isentropes));    
    
    % Isentropes
    if strcmp(options.isentropes, 'on')==1 % plot_isentropes == 1
        for i=1:(n_s)
            for ii=1:n_p_isentropes
                p_isentropes(:,i) = p_display_disc;
                try
                    h_isentropes(ii,i) = refpropm_v1('h', 'p', p_display_disc(ii), 's', s_isentrope(i), medium, composition);                 
                catch
                    h_isentropes(ii,i) = NaN;
                end
            end
        end 
    end
%% Calculate lines of constant vapor fraction
    x_vapor = [0.2 0.4 0.6 0.8];
    
    % Number of points for calculation of enthalpies for lines of constant vapor quality
    n_p_constvapor = 10;
    
    p_disc_reduced = linspace(p_display(1,1), max(p_sat), n_p_constvapor);

    % Preallocate space
    h_const_vapor = zeros(n_p_constvapor, length(x_vapor));

    if strcmp(options.isoqualities, 'on')==1 % plot_isoquality == 1
        for i=1:length(x_vapor)
            for ii=1:n_p_constvapor-1
                try
                    h_const_vapor(ii,i) = refpropm_v1('h', 'p', p_disc_reduced(ii), 'Q', x_vapor(i), medium, composition);
                catch
                    h_const_vapor(ii,i) = 0;
                end
            end
            % Add values at critical point
            p_disc_reduced(n_p_constvapor) = p_crit;
            h_const_vapor(n_p_constvapor, i) = h_crit;

        end

        h_const_vapor_spline = zeros(length(p_sat_total), length(x_vapor)); 
        for i=1:length(x_vapor)
            h_const_vapor_spline(:,i) = spline(p_disc_reduced, h_const_vapor(:,i), p_sat_total);
        end
    end
    
            
%% --------------------------------------------------------------------- %%
%                             Plot Figure                                 %
%  ---------------------------------------------------------------------  %
%  Define inputs for figure
figure_width = options.width;       % cm
figure_height = options.height;      % cm

fs = options.fs;                % Fontsize
font = options.font; % Fontstyle

  % --------------------------------------------------------------------------------------
  
figure_logph = figure('Units','Centimeters', 'Name', 'log(p)-h-Diagram',...
                   'Position', [10 5 figure_width figure_height], ...
                   'PaperSize', [figure_width figure_height], ...
                   'PaperPosition', [10 5 figure_width figure_height]);

    axes_ph = axes('Parent', figure_logph,'YScale','log', ...
         'FontName', 'Cambria', 'ylim', [p_display(1,1)/100 p_display(1,2)/100], 'xlim', [h_display(1,1) h_display(1,2)]/1000,...
         'yTick', [1 2 3 4 5 6 7 8 9 10 20 30 40 50 60 70 80 90 100], 'yTickLabel',...
         {'1','','','','5','','','','','10','','','','50','','','','','100'},...
         'FontSize', fs, 'FontName', font);  % {'1','2','3','4','5','6','7','8','9','10','20','30','40','50','60','70','80','90','100'}
         hold on;

if strcmp(options.box, 'on')==1
    box on;
end
if strcmp(options.grid, 'on')==1
    grid on;
end
         
    % Plot Vapor Dome     
    semilogy(h_bubble/1000, p_sat/100, 'LineWidth', 0.2,  'Color', [0.38 0.38 0.38])%, ...
    semilogy(h_dew/1000, p_sat/100, 'LineWidth', 0.2,  'Color', [0.38 0.38 0.38])
    semilogy(h_sat_crit/1000, p_sat_crit_spline/100, 'LineWidth', 0.2,  'Color', [0.38 0.38 0.38])
    
    % Plot Isothermals
    if strcmp(options.isothermals, 'on')==1
%         for i=1:(n_T*2+1)
%             semilogy(h_isothermals(:,i)/1000, p_display_disc/100, 'LineWidth', 0.05, 'Color',[0.8 0.8 0.8]);
        for i=1:(n_T)
            semilogy(h_isothermals(:,i)/1000, p_isothermals(:,i)/100, 'LineWidth', 0.05, 'Color',[0.8 0.8 0.8]);
        end
    end
    
    % Plot Isentropes
    if strcmp(options.isentropes, 'on')==1
        for i=1:(n_s)
            semilogy(h_isentropes(:,i)/1000, p_isentropes(:,i)/100, 'LineWidth', 0.05, 'Color',[0.7 1 1], 'LineStyle', '--');
        end    
    end
    
    % Lines of Constant Vapour Quality
    if strcmp(options.isoqualities, 'on')==1
        for i=1:length(x_vapor)
            semilogy(h_const_vapor_spline(:,i)/1000, p_sat_total/100, 'LineWidth', 0.01, 'Color', [0.95 0.95 0.95]); 
        end
    end
%     for i=1:length(x_vapor)
%         h_const_vapor_spline(:,i) = spline(p_disc_reduced, h_const_vapor(:,i), p_sat_total);
%     end    
    
    % Plot Cycle
    semilogy(h_cycle/1000, p_cycle/100, 'Linewidth', 1, 'Color', 'b');
     
    if strcmp(options.marker, 'on')==1
        semilogy(cycle.h/1000, cycle.p/100, 'Linewidth', 1, 'Color', 'b', 'Marker', 'x');
    end        
    
    % contour(h_display_disc/1000, p_display_disc/100, T_field_disc, T_isothermal); 
    % legend('Working Fluid', 'Sink', 'Location', 'NorthEast');

    y1=ylabel('log(p) (bar)','Rotation',90, 'FontSize', fs, 'FontName', font);
    x1=xlabel('Specific Enthalpy (kJ/kg)', 'FontSize', fs, 'FontName', font);  

end
