function [Exergy] = HP_ihx_ExergyAnalysis(boundaryCond, cycleInput, componentInput, summary, cycle)
%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
% -------------------------------------------------------------------------
%% Script conducts an exergy analysis for a simple heat pump cycle
%  The model uses the already calculated state points of the cycle
%  Output:      The output is a complete exergy analysis of the IHX cycle
% Limitation:   The implementation considers all state points to be above
%               the dead state
%%  -----------------------------------------------------------------------
%  -> Numbering of components:
%     I:   Compressor
%     II:  HX Heat Sink
%     III: Internal HX
%     IV:  Throttling Valve (Dissipative Component -> no product)
%     V:   HX Heat Source
%  ------------------------------------------------------------------------
%  -> Numbering of state points/streams
%
%     Note: - In- and Outlet always in perspective of considered stream
%           - Heat exchanger in counterflow (inlet of stream 1 is on the
%             same side as outlet of stream 2)
%
%     HP-Cycle:  
%     1:   Compressor Inlet
%     2:   Compressor Outlet/Desuperheater Inlet
%     3:   Desuperheater Outlet/Condenser Inlet
%     4:   Condenser Outlet/Subcooler Inlet
%     5:   Subcooler Outlet/InternalHX Inlet HotSide
%     6:   InternalHX Outlet HotSide/ Throttling Valve Inlet
%     7:   Throttling Valve Outlet/Evaporator Inlet
%     8:   Evaporator Outlet/InternalHX Inlet ColdSide
%     1:   InternalHX Outlet ColdSide/Compressor Inlet
%
%     Heat Source:
%     21:  Heat Source Inlet -> Inlet Evaporator
%     22:  Heat Source Outlet -> Outlet Evaporator
%
%     Heat Sink:
%     31:  Heat Sink Inlet -> Inlet of Subcooler
%     32:  Outlet of Subcooler/Inlet of Condenser
%     33:  Outlet of Condenser/Inlet of Desuperheater
%     34:  Heat Sink Outlet -> Outlet of Desuperheater
%  ------------------------------------------------------------------------

%% Processing input variables
%  Boundary conditions
    medium_source               = boundaryCond.medium_source;
    medium_sink                 = boundaryCond.medium_sink;
    p_source                    = boundaryCond.p_source;
    p_sink                      = boundaryCond.p_sink;
    T_source_in                 = boundaryCond.T_source_in;
    T_source_out                = boundaryCond.T_source_out;
    T_sink_in                   = boundaryCond.T_sink_in;
    T_sink_out                  = boundaryCond.T_sink_out;
    h_source                    = summary.Component.HX.h_source;
    h_sink                      = summary.Component.HX.h_sink;
    m_flow_source               = summary.Performance.m_flow_soure;
    m_flow_sink                 = summary.Performance.m_flow_sink;
    
%  Cycle Input
    medium                      = cycleInput.medium;
    composition                 = cycleInput.composition;
    p                           = cycle.p;
    h                           = cycle.h;
    s                           = cycle.s;
    m_flow                      = cycle.m_flow;
    W_dot                       = summary.EB.W_dot;
    Q_dot_source                = summary.EB.Q_dot_source;
    Q_dot_sink                  = summary.EB.Q_dot_sink;
    COP                         = summary.Performance.COP;
    
% Component Input
    delta_T_pinch_source_min    = componentInput.HX.delta_T_pinch_source_min;
    delta_T_pinch_sink_min      = componentInput.HX.delta_T_pinch_sink_min;

%% Ambient Conditions
    T_amb = boundaryCond.T_amb;         % [K]     % Ambient Temperature
    p_amb = boundaryCond.p_amb;         % [Pa]    % Ambient Pressure  %1.01325e5;   
    
%% Reference States
    % Cycle Medium
    h_0_cycle = refpropm_v1('h', 'T', T_amb, 'p', p_amb/1000, medium, composition);
    s_0_cycle = refpropm_v1('s', 'T', T_amb, 'p', p_amb/1000, medium, composition);
    
    % Source Medium
    h_0_source = CoolProp.PropsSI('H', 'T', T_amb, 'P', p_amb, medium_source);
    s_0_source = CoolProp.PropsSI('S', 'T', T_amb, 'P', p_amb, medium_source);    
    
    % Sink Medium
    h_0_sink = CoolProp.PropsSI('H', 'T', T_amb, 'P', p_amb, medium_sink);    
    s_0_sink = CoolProp.PropsSI('S', 'T', T_amb, 'P', p_amb, medium_sink); 
    
%% Calculating Entropies
    
    % Source
    s_21 = CoolProp.PropsSI('S', 'H', h_source(end), 'P', p_source, medium_source);
    s_22 = CoolProp.PropsSI('S', 'H', h_source(1), 'P', p_source, medium_source);    
    
    % Sink
    s_31 = CoolProp.PropsSI('S', 'H', h_sink(end), 'P', p_sink, medium_sink);
    s_34 = CoolProp.PropsSI('S', 'H', h_sink(1), 'P', p_sink, medium_sink);   
    
%% Calculating Exergy Flows

    % Cycle:
    E_dot = NaN(length(h),1);
    for i=1:length(h)
        E_dot(i) = m_flow * (h(i)-h_0_cycle - T_amb*(s(i)-s_0_cycle));
    end 
     
    % Source    
    E_dot_21 = m_flow_source * (h_source(end)-h_0_source - T_amb*(s_21-s_0_source)); 
    E_dot_22 = m_flow_source * (h_source(1)-h_0_source - T_amb*(s_22-s_0_source)); 
    
    % Sink    
    E_dot_31 = m_flow_sink * (h_sink(end)-h_0_sink - T_amb*(s_31-s_0_sink)); 
    E_dot_34 = m_flow_sink * (h_sink(1)-h_0_sink - T_amb*(s_34-s_0_sink));     
    
%% Exergy Balances
%     Note: - No Exergy Losses (Fuel is the difference 21-23, Stream 23 not
%             considered as input and thus, not as loss)

% Exergy Fuel and Product
    % I:   Compressor
    E_dot_p(1) = E_dot(2)-E_dot(1);
    E_dot_f(1) = W_dot;
    
    % II:  HX Heat Sink
    E_dot_p(2) = E_dot_34-E_dot_31;
    E_dot_f(2) = E_dot(2)-E_dot(5);    

    % III: Internal HX
    E_dot_p(3) = E_dot(1)-E_dot(8);
    E_dot_f(3) = E_dot(5)-E_dot(6);      
    
    % IV: Throttling Valve (dissipative component -> no product)
    E_dot_p(4) = 0;
    E_dot_f(4) = E_dot(6)-E_dot(7);  
    
    % V:  HX Heat Source 
    E_dot_p(5) = E_dot(8)-E_dot(7);
    E_dot_f(5) = E_dot_21-E_dot_22;    
    
    % System:
    E_dot_p_total = E_dot_p(2);
    E_dot_f_total = E_dot_f(1)+E_dot_f(5);
    
% Exergy Destruction
    % Components
    E_dot_d = E_dot_f-E_dot_p;
    
    % System
    E_dot_d_total = E_dot_f_total-E_dot_p_total;
%     E_dot_d_total_sum = sum(E_dot_d);
    
%% Exergetic Performance Indicators
    % Exergetic efficiency
    eps = E_dot_p./E_dot_f;
    eps_total = E_dot_p_total/E_dot_f_total;
    
    y_destr_ast = E_dot_d./E_dot_d_total;

%% Calculation of temperature glide matching indicator

% Avarage thermodynamic temperatures
% Source
T_source_av = (T_source_in - T_source_out)/log(T_source_in/T_source_out);
T_evap_ideal_av = (T_source_in - T_source_out)/log((T_source_in-delta_T_pinch_source_min)/(T_source_out-delta_T_pinch_source_min));
T_evap_av = (h(8)-h(7)) / (s(8)-s(7));
delta_T_source_av = T_source_av-T_evap_av;

% Sink
T_sink_av = (T_sink_out-T_sink_in)/log(T_sink_out/T_sink_in);
T_cond_ideal_av = (T_sink_out-T_sink_in)/log((T_sink_out+delta_T_pinch_sink_min)/(T_sink_in+delta_T_pinch_sink_min));
T_cond_av = (h(2)-h(5)) / (s(2)-s(5));
delta_T_sink_av = T_cond_av-T_sink_av;

% Exergy destruction contributions
% Source
E_dot_d_source_pinch = Q_dot_source * T_amb * (T_source_av - T_evap_ideal_av)/(T_source_av * T_evap_ideal_av);
E_dot_d_source_fluid = E_dot_d(4)-E_dot_d_source_pinch;

% Sink
E_dot_d_sink_pinch = Q_dot_sink * T_amb * (T_cond_ideal_av-T_sink_av)/(T_cond_ideal_av*T_sink_av);
E_dot_d_sink_fluid = E_dot_d(2)-E_dot_d_sink_pinch;    

% Glide matching indicators
pi_source = E_dot_d(5)/E_dot_d_source_pinch;
pi_sink   = E_dot_d(2)/E_dot_d_sink_pinch;

y_destr_fluid_source_ast = E_dot_d_source_fluid/E_dot_d_total; 
y_destr_fluid_sink_ast   = E_dot_d_sink_fluid/E_dot_d_total;
y_destr_pinch_source_ast = E_dot_d_source_pinch/E_dot_d_total; 
y_destr_pinch_sink_ast   = E_dot_d_sink_pinch/E_dot_d_total;

% y_destr_fluid_source_p = E_dot_d_source_fluid/E_dot_p_total; 
% y_destr_fluid_sink_p   = E_dot_d_sink_fluid/E_dot_p_total;

%%  Lorentz COP 
COP_Lorenz = T_sink_av/(T_sink_av-T_source_av);
eta_Lorenz = COP/COP_Lorenz;

%% Carnot COP
COP_Carnot = T_sink_out/(T_sink_out-T_source_out);
eta_Carnot = COP/COP_Carnot;

%% Summarizing Results
Exergy.E_dot_p                  = E_dot_p;
Exergy.E_dot_f                  = E_dot_f;
Exergy.E_dot_d                  = E_dot_d;
Exergy.eps                      = eps;
Exergy.E_dot_p_total            = E_dot_p_total;
Exergy.E_dot_f_total            = E_dot_f_total;
Exergy.E_dot_d_total            = E_dot_d_total;
Exergy.eps_total                = eps_total;
Exergy.y_destr_ast              = y_destr_ast;
Exergy.E_dot                    = E_dot;
Exergy.E_dot_source             = [E_dot_21 E_dot_22];
Exergy.E_dot_sink               = [E_dot_31 E_dot_34];

% Glide matching indicators
Exergy.E_dot_d_source_pinch     = E_dot_d_source_pinch;
Exergy.E_dot_d_source_fluid     = E_dot_d_source_fluid;
Exergy.E_dot_d_sink_pinch       = E_dot_d_sink_pinch;
Exergy.E_dot_d_sink_fluid       = E_dot_d_sink_fluid;
Exergy.pi_source                = pi_source;
Exergy.pi_sink                  = pi_sink;

Exergy.y_destr_fluid_source_ast = y_destr_fluid_source_ast; 
Exergy.y_destr_fluid_sink_ast   = y_destr_fluid_sink_ast;
Exergy.y_destr_pinch_source_ast = y_destr_pinch_source_ast; 
Exergy.y_destr_pinch_sink_ast   = y_destr_pinch_sink_ast;

% Temperatures
Exergy.T_source_av              = T_source_av;
Exergy.T_evap_av                = T_evap_av;
Exergy.delta_T_source_av        = delta_T_source_av;

Exergy.T_sink_av                = T_sink_av;
Exergy.T_cond_av                = T_cond_av;
Exergy.delta_T_sink_av          = delta_T_sink_av;

% Efficiencies
Exergy.COP_Carnot               = COP_Carnot;
Exergy.COP_Lorenz               = COP_Lorenz;
Exergy.eta_Carnot               = eta_Carnot;
Exergy.eta_Lorenz               = eta_Lorenz;

% Dead State
Exergy.T_amb                    = T_amb;
Exergy.p_amb                    = p_amb;

end
