function [COP, summary, cycle] = HP_ihx_cycle(x, boundaryCond, cycleInput, componentInput, solverInput, economyInput)
%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
% -------------------------------------------------------------------------

%% Heat Pump Model with internal heat exchanger (IHX)
%  Input variables defined by calling script
%  Defined as algorithm

%% Input Variables
%  - Boundary Conditions
%  - Assumptions for HP Calculation

%%  -----------------------------------------------------------------------
%  -> Numbering of components:
%     I:   Compressor
%     II:  HX Heat Sink
%     III: Internal HX
%     IV:  Throttling Valve (Dissipative Component -> no product)
%     V:   HX Heat Source
%  ------------------------------------------------------------------------
%  -> Numbering of state points/streams
%
%     Note: - In- and Outlet always in perspective of considered stream
%           - Heat exchanger in counterflow (inlet of stream 1 is on the
%             same side as outlet of stream 2)
%
%     HP-Cycle:  
%     1:   Compressor Inlet
%     2:   Compressor Outlet/Desuperheater Inlet
%     3:   Desuperheater Outlet/Condenser Inlet
%     4:   Condenser Outlet/Subcooler Inlet
%     5:   Subcooler Outlet/InternalHX Inlet HotSide
%     6:   InternalHX Outlet HotSide/ Throttling Valve Inlet
%     7:   Throttling Valve Outlet/Evaporator Inlet
%     8:   Evaporator Outlet/InternalHX Inlet ColdSide
%     1:   InternalHX Outlet ColdSide/Compressor Inlet
%
%     Heat Source:
%     21:  Heat Source Inlet -> Inlet Evaporator
%     22:  Heat Source Outlet -> Outlet Evaporator
%
%     Heat Sink:
%     31:  Heat Sink Inlet -> Inlet of Subcooler
%     32:  Outlet of Subcooler/Inlet of Condenser
%     33:  Outlet of Condenser/Inlet of Desuperheater
%     34:  Heat Sink Outlet -> Outlet of Desuperheater

tic;

% %% Clear old variables which might influence the solving process
% clear x Jac delta_T_sink delta_T_source Res RelRes

%% Inputs
p_evap = x(1);                  % Evaporation pressure  [kPa]
p_cond = x(2);                  % Condensation pressure [kPa]

%% Processing input variables
%  Boundary conditions
T_source_in                 = boundaryCond.T_source_in;
T_source_out                = boundaryCond.T_source_out;
p_source                    = boundaryCond.p_source;
medium_source               = boundaryCond.medium_source;
T_sink_in                   = boundaryCond.T_sink_in;
T_sink_out                  = boundaryCond.T_sink_out;
p_sink                      = boundaryCond.p_sink;
medium_sink                 = boundaryCond.medium_sink;
% Q_dot_sink                  = boundaryCond.Q_dot_sink;

%  Component Inputs
%  HX
n_states_evap               = componentInput.HX.n_states_evap;
n_states_sh                 = componentInput.HX.n_states_sh;
n_states_cond               = componentInput.HX.n_states_cond;
n_states_desup              = componentInput.HX.n_states_desup;
n_states_sub                = componentInput.HX.n_states_sub;
n_states_ihx                = componentInput.HX.n_states_ihx;
delta_T_pinch_source_min    = componentInput.HX.delta_T_pinch_source_min;
delta_T_pinch_sink_min      = componentInput.HX.delta_T_pinch_sink_min;
delta_T_pinch_ihx_min       = componentInput.HX.delta_T_pinch_ihx_min;
%  Comp
n_states_comp               = componentInput.Comp.n_states_comp;
eta_is_comp_design          = componentInput.Comp.eta_is_comp_design;
heatloss                    = componentInput.Comp.heatloss;
eta_motor                   = componentInput.Comp.eta_motor;

%  Cycle Input
delta_T_minSH               = cycleInput.delta_T_minSH;
dp_evap                     = cycleInput.dp_evap;
dp_sh                       = cycleInput.dp_sh;
dp_desup                    = cycleInput.dp_desup;
dp_cond                     = cycleInput.dp_cond;
dp_sub                      = cycleInput.dp_sub;
dp_ihx_hot                  = cycleInput.dp_ihx_hot;  
dp_ihx_cold                 = cycleInput.dp_ihx_cold;  
Q_evap_out                  = cycleInput.Q_evap_out;
medium                      = cycleInput.medium;
composition                 = cycleInput.composition;

%  Solver Input
limit_sc                    = solverInput.limit_sc;
show_info                   = solverInput.show_info;
tol_delta_T                 = solverInput.tol_delta_T;
t_max                       = solverInput.t_max;

%% Preallocate Space for variables
T = NaN(1,8);
h = NaN(1,8);
s = NaN(1,8);
T_evap              = zeros(n_states_evap,1);
T_source            = zeros(n_states_evap,1);
T_sink              = zeros(n_states_sub+n_states_cond+n_states_desup-2,1);
T_cond              = zeros(n_states_sub+n_states_cond+n_states_desup-2,1);
T_ihx_cold          = NaN(n_states_ihx, 1);
T_ihx_hot           = NaN(n_states_ihx, 1);
p_evap_vector       = NaN(n_states_evap,1);
p_cond_vector       = NaN(n_states_desup+n_states_cond+n_states_sub-2,1);
p_ihx_cold_vector   = NaN(n_states_ihx,1);
p_ihx_hot_vector    = NaN(n_states_ihx,1);

delta_T_lm_source   = NaN(n_states_evap-1,1);
UA_source           = NaN(n_states_evap-1,1);

delta_T_lm_ihx      = NaN(n_states_ihx-1,1);
UA_ihx              = NaN(n_states_ihx-1,1);

delta_T_lm_sink     = NaN(n_states_sub+n_states_cond+n_states_desup-2-1,1);
UA_sink             = NaN(n_states_sub+n_states_cond+n_states_desup-2-1,1);

%% Calculate fixed variables
T_21 	= T_source_in;
h_21	= CoolProp.PropsSI('H', 'T', T_source_in, 'P', p_source, medium_source);

T_31    = T_sink_in; 
T_34    = T_sink_out;
h_31    = CoolProp.PropsSI('H', 'T', T_sink_in,'P', p_sink,  medium_sink);
h_34    = CoolProp.PropsSI('H', 'T', T_sink_out,'P', p_sink,  medium_sink);

%% Preparing variables
% Critical Properties
% Call SATSPLN
Run_SATSPLN=1;    % if = 1, SATSPLN is called
save('C:\Program Files\REFPROP\Run_SATSPLN', 'Run_SATSPLN');
[T_crit, p_crit, h_crit, s_crit]  = refpropm_v1('Tphs', 'c', 0,'',0, medium, composition);
Run_SATSPLN=0;
save('C:\Program Files\REFPROP\Run_SATSPLN', 'Run_SATSPLN');

% Calculate pressures in sink and source considering pressure drop on refrigerant side
    p_evap_vector(1) = p_evap;
    for i = 1:length(dp_evap)
        p_evap_vector(i+1) = p_evap - (sum(dp_evap(1:i)));
    end   

    % Desuperheater + Condenser + Subcooler
    p_cond_vector(n_states_desup) = p_cond;
    for i=1:length(dp_desup)
        p_cond_vector(n_states_desup-i) = p_cond_vector(n_states_desup) + sum(dp_desup(n_states_desup-i:end)); %(n_states_desup-1-i:end)
    end
    for i = 1:n_states_cond-1
        p_cond_vector(i+n_states_desup) = p_cond_vector(n_states_desup) - sum(dp_cond(1:i));
    end
    for i = 1:n_states_sub-1
        p_cond_vector(i+n_states_desup+n_states_cond-1) = p_cond_vector(n_states_desup+n_states_cond-1) - sum(dp_sub(1:i));
    end

    % Internal Heat Exchanger
    p_ihx_cold_vector(1) = p_evap_vector(end);
    for i = 1:n_states_ihx-1 
        p_ihx_cold_vector(i+1) = p_evap_vector(end) - sum(dp_ihx_cold(1:i));
    end
    p_ihx_hot_vector(1) = p_cond_vector(end);
    for i = 1:n_states_ihx-1 
        p_ihx_hot_vector(i+1) = p_cond_vector(end) - sum(dp_ihx_hot(1:i));
    end    
    
    % Define pressures of cycle state points 
    p(1) = p_ihx_cold_vector(end);
    p(2) = p_cond_vector(1);
    p(3) = p_cond;
    p(4) = p_cond_vector(n_states_desup+n_states_cond-1);
    p(5) = p_cond_vector(end);
    p(6) = p_ihx_hot_vector(end);
    p(7) = p_evap;
    p(8) = p_evap_vector(end);
 
%% Cycle Model
%% Heat Source Heat Exchanger
% Evaporator Outlet
[h(8), T(8)]    = refpropm_v1('ht', 'p', p_evap, 'Q', Q_evap_out, medium, composition);

%% State 4: Outlet of Condenser
% Define state 4
if p_cond < p_crit * limit_sc % 1 == strcmp('Sub', SubTrans)  %
    % Subcritical Operation
    % Outlet of condenser at saturation line
    [h(4),T(4)] = refpropm_v1('ht', 'p', p_cond, 'Q', 0, medium, composition);
else
    % Transcritical Operation
    % Condenser outlet at pinch point temperature difference
    T(4) = T_sink_in + delta_T_pinch_sink_min;
    h(4) = refpropm_v1('h', 'T', T(4), 'p', p_cond, medium, composition);
end

%% Heat Sink Heat Exchanger

% Subcooler Outlet -> Subcooling to pinch -> requires further investigation
T(5)            = min(T(4), T_sink_in + delta_T_pinch_sink_min);

% State 4 = state 5 for transcritical operation
if T(4) == T(5)
    h(5)      = h(4);
else
    h(5)      = refpropm_v1('h', 'T', T(5), 'p', p_cond, medium, composition);
end

if p_cond<p_crit && T(4) ==T(5) && show_info==1
    display('Warning: No subcooling')
end

%% Iteration around internal HX to set pinch
% if iter==1
%     delta_T_SH_IHX = 5;
% end
delta_T_SH_IHX = T_sink_in - T(8);

iter_IHX = 1;
error_delta_T_pinch_ihx=1;
while abs(error_delta_T_pinch_ihx)>tol_delta_T
    % Evaporator Outlet/Compressor Inlet
    T(1)            = T(8)+delta_T_SH_IHX;
    h(1)            = refpropm_v1('h', 'T', T(1), 'p', p_evap, medium, composition);
    
    % Internal Heat Exchanger
    h(6) = h(5)-(h(1)-h(8));
    T(6) = refpropm_v1('T', 'h', h(6),  'p', p_cond, medium, composition);
    
    % Discretization
    h_ihx_cold = linspace(h(8), h(1), n_states_ihx);
    h_ihx_hot = linspace(h(5), h(6), n_states_ihx);
    
    for i_ihx=1:n_states_ihx   % parfor
        try
            T_ihx_cold(i_ihx)        	= refpropm_v1('T', 'h', h_ihx_cold(i_ihx), 'p', p_ihx_cold_vector(i_ihx), medium, composition);
        catch
            T_ihx_cold(i_ihx)           = NaN;
        end
        try
            T_ihx_hot(i_ihx)           = refpropm_v1('T', 'h', h_ihx_hot(i_ihx), 'p', p_ihx_hot_vector(i_ihx), medium, composition);
        catch
            T_ihx_hot(i_ihx)           = NaN;
        end
    end
    
    T_ihx_hot_ud = flipud(T_ihx_hot);

    delta_T_ihx = T_ihx_hot_ud-T_ihx_cold;
    error_delta_T_pinch_ihx = delta_T_pinch_ihx_min - min(delta_T_ihx);
    delta_T_SH_IHX = delta_T_SH_IHX - error_delta_T_pinch_ihx;
    iter_IHX = iter_IHX+1;
    if iter_IHX>100
        error(strcat('IHX did not converge: ', num2str(error_delta_T_pinch_ihx), ' K'));
    end
end


%% Compressor
% Calculate efficiency
switch cycleInput.Define_Efficiency 
    case 'constant'
        eta_is_comp = eta_is_comp_design;
    case 'BuiltInPressureRatio'
        kappa_comp = refpropm_v1('k', 'T', boundaryCond.T_source_in, 'Q', 1, medium, composition);
        eta_is_comp = GranrydEfficiency(componentInput.Comp.pi_comp, kappa_comp, p(2)/p(1)) * eta_is_comp_design;
    otherwise
        disp('Defined compressor efficiency not implemented')
end

if p_cond < p_crit * limit_sc % 1 == strcmp('Sub', SubTrans)  %
    % Subcritical Operation
    % Disctinction in Desuperheating, Condensing and Subcooling for Subcritical Process
    [h(3),T(3)] = refpropm_v1('ht', 'p', p_cond, 'Q', 1, medium, composition);
    
    % Iteration to ensure dry compression at outlet
    delta_T_SH = 0;
    error_delta_T_SH = 10;
    iter_SH=1;
    while error_delta_T_SH>= tol_delta_T     %abs(min(delta_T_SH_summary))<=delta_T_minSH
        % If the superheating from the IHX still yields a "wet compression", it
        % is assumed that the fluid is superheated from T(1) to T_1_SH by
        % an electrical heater or similar
        % it basically means, that the efficiency is decreased to obtain a dry compression
        
        % Compressor
        T_1_SH = T(1)+delta_T_SH;
        
        h_1_SH = max(refpropm_v1('h',  'T', T_1_SH, 'p', p_evap, medium, composition), h(7));  % <- Avoids that enthalpy in liquid phase is calculated due to small numerical tolerances
        s_1_SH = refpropm_v1('s',  'p', p_evap, 'h', h_1_SH, medium, composition);
        
        h_2_is = refpropm_v1('h', 'p', p_cond, 's', s_1_SH, medium, composition);
        
        h_2_adiabat = h(1) + (h_2_is-h_1_SH)/eta_is_comp;
        
        h(2) = h_2_adiabat - heatloss * (h_2_adiabat-h_1_SH);
        
        T(2) = refpropm_v1('T', 'h', h(2), 'p', p_cond, medium, composition);
        
        delta_T_SH_summary = [T_1_SH-T(8) T(2)-T(3)];
        
        error_delta_T_SH = delta_T_minSH-min(delta_T_SH_summary);
        
        delta_T_SH = max(delta_T_SH + error_delta_T_SH, 0);  % <- numerical inaccuracies (<1e-10) can cause negative values, which cause bad solving behaviour
        if toc > t_max
            display(strcat({'SH did not converge within '}, num2str(t_max), ' s'))
            fejl(0)
            %MException('SH did not converge within 60 s')
        end
        iter_SH=iter_SH+1;
    end
    
else
    % Transcritical
    T_1_SH = T(1)+delta_T_minSH;
    [h_1_SH s_1_SH]= refpropm_v1('hs',  'T', T_1_SH, 'p', p_evap, medium, composition);
    
    h_2_is = refpropm_v1('h', 'p',p_cond, 's', s_1_SH, medium, composition);
    
    h_2_adiabat = h(1) + (h_2_is-h_1_SH)/eta_is_comp;
    h(2) = h_2_adiabat - heatloss * (h_2_adiabat-h_1_SH);
    
    T(2) = refpropm_v1('T', 'h', h(2), 'p', p_cond, medium, composition);
    
    % Equidistant Discretization for Transcritical Process
    h(3) = h(2);
    T(3) = T(2);
end

rho_1 = refpropm_v1('d', 'h', h_1_SH, 'p', p_evap, medium, composition);

%% Define Load
switch cycleInput.DefineLoad
    case 'Q_dot_sink'
        m_flow          = cycleInput.Q_dot_sink/(h(2)-h(5));
        V_dot_comp_in   = m_flow/rho_1; 
        V_dot_comp_disp = V_dot_comp_in / componentInput.Comp.eta_vol_comp;      
    case 'Q_dot_source'
        m_flow          = cycleInput.Q_dot_source/(h(1)-h(5));
        V_dot_comp_in   = m_flow/rho_1; 
        V_dot_comp_disp = V_dot_comp_in / componentInput.Comp.eta_vol_comp;        
    case 'V_dot_comp_in'
        V_dot_comp_in   = cycleInput.V_dot_comp_in;
        V_dot_comp_disp = V_dot_comp_in / componentInput.Comp.eta_vol_comp;
        m_flow          = V_dot_comp_in * rho_1;   
    case 'V_dot_comp_disp'
        V_dot_comp_disp = cycleInput.V_dot_comp_disp;
        V_dot_comp_in   = V_dot_comp_disp * componentInput.Comp.eta_vol_comp;
        m_flow          = V_dot_comp_in * rho_1;   
    otherwise
        display('Definition of cycle load not implemented')
end

V_dot(1) = V_dot_comp_in;

%% Internal Heat Exchanger: Heat Load
Q_dot_ihx = m_flow * (h(8)-h(1));
Q_dot_ihx_cum = m_flow*(h_ihx_cold-h(8));

% Calculate delta_T_lm and UA
for i=1:(length(h_ihx_cold)-1)
    if ((T_ihx_hot_ud(i)-T_ihx_cold(i)) / (T_ihx_hot_ud(i+1)-T_ihx_cold(i+1)))==1
        delta_T_lm_ihx(i) = T_ihx_hot_ud(i)-T_ihx_cold(i);
    else
        delta_T_lm_ihx(i) = ((T_ihx_hot_ud(i)-T_ihx_cold(i)) - (T_ihx_hot_ud(i+1)-T_ihx_cold(i+1))) / log((T_ihx_hot_ud(i)-T_ihx_cold(i)) / (T_ihx_hot_ud(i+1)-T_ihx_cold(i+1)));
    end
    UA_ihx(i) = (Q_dot_ihx_cum(i+1)-Q_dot_ihx_cum(i)) / delta_T_lm_ihx(i);
end

UA_ihx_total = nansum(UA_ihx); %sum(max(UA_sink,0));%
delta_T_ihx_average = Q_dot_ihx/UA_ihx_total;

%% Sink Side
% Heat Flow Rates
Q_dot_desup     = m_flow*(h(2)-h(3));
Q_dot_cond      = max(0,m_flow*(h(3)-h(4)));
Q_dot_subcool   = max(0,m_flow*(h(4)-h(5)));
Q_dot_sink_off  = m_flow*(h(2)-h(5));

% Secondary Side
m_flow_sink     = Q_dot_sink_off/(h_34-h_31);
if p_cond <= (p_crit * limit_sc) % 1 == strcmp('Sub', SubTrans)  %
    h_32            = h_31 + Q_dot_subcool/m_flow_sink;
    h_33            = h_31 + (Q_dot_subcool+Q_dot_cond)/m_flow_sink;
    
    T_32            = CoolProp.PropsSI('T', 'H', h_32, 'P', p_sink,  medium_sink);
    T_33            = CoolProp.PropsSI('T', 'H', h_33, 'P', p_sink,  medium_sink);
else
    h_32            = h_31 ;
    h_33            = h_34 ;
    
    T_32            = T_31;
    T_33            = T_34;
end

% Calculation of discretized states
if p_cond < p_crit % 1 == strcmp('Sub', SubTrans)%
    % Subcritical operation
    h_cond_desup = linspace(h(2), h(3), n_states_desup);
    h_sink_desup = linspace(h_34, h_33, n_states_desup);
    
    h_cond_cond = linspace(h(3), h(4), n_states_cond);
    h_sink_cond = linspace(h_33, h_32, n_states_cond);
    
    h_cond_sub = linspace(h(4), h(5), n_states_sub);
    h_sink_sub = linspace(h_32, h_31, n_states_sub);   

    h_cond          = [h_cond_desup(1:end-1), h_cond_cond(1:end-1), h_cond_sub]';
    h_sink          = [h_sink_desup(1:end-1), h_sink_cond(1:end-1), h_sink_sub]';
    
    T_cond(1)     = T(2);
    T_sink(1)     = T_34;
    
    T_cond(end)     = T(5);
    T_sink(end)     = T_31;
    
    for i=2:length(h_cond)-1
        try
            T_cond(i)       = refpropm_v1('T', 'h', h_cond(i),  'p', p_cond_vector(i), medium, composition);
        catch
            T_cond(i)       = NaN;
        end
        try
            T_sink(i)       = CoolProp.PropsSI('T', 'H', h_sink(i), 'P', p_sink,  medium_sink);
        catch
            T_sink(i)       = NaN;
        end
    end
else
    h_cond          = linspace(h(2), h(5), n_states_sub+n_states_cond+n_states_desup-2)';
    h_sink          = linspace(h_34, h_31, n_states_sub+n_states_cond+n_states_desup-2)';
    
    T_cond(1)       = T(2);
    T_sink(1)       = T_34;
    
    T_cond(end)     = T(5);
    T_sink(end)     = T_31;
    
    for i=1:length(h_cond)
        try
            T_cond(i)       = refpropm_v1('T', 'h', h_cond(i),  'p', p_cond_vector(i), medium, composition);
        catch
            T_cond(i)       = NaN;
        end
        try
            T_sink(i)       = CoolProp.PropsSI('T', 'H', h_sink(i),  'P', p_sink,  medium_sink);
        catch
            T_sink(i)       = NaN;
        end
    end
end

% Temperature Differences
delta_T_sink        = T_cond-T_sink;
Q_dot_sink_cum      = m_flow * (h(2) - h_cond);

% Calculate UA
for i=1:(length(h_cond)-1)
    if (T_cond(i)-T_sink(i)) / (T_cond(i+1)-T_sink(i+1))==1
        delta_T_lm_sink(i) = T_cond(i)-T_sink(i);
    else
        delta_T_lm_sink(i) = ((T_cond(i)-T_sink(i)) - (T_cond(i+1)-T_sink(i+1))) / log((T_cond(i)-T_sink(i)) / (T_cond(i+1)-T_sink(i+1)));
    end
    UA_sink(i) = (Q_dot_sink_cum(i+1)-Q_dot_sink_cum(i)) / delta_T_lm_sink(i);
end

UA_sink_total = nansum(UA_sink); %sum(max(UA_sink,0));%
delta_T_sink_average = Q_dot_sink_off/UA_sink_total;


%% Throttling Valve
h(7)                = h(6);
T(7)                = refpropm_v1('T', 'h', h(7), 'p', p_evap, medium, composition);

%% Heat Source Heat Exchanger

Q_dot_evap          = m_flow*(h(8)-h(7));
Q_dot_source        = Q_dot_evap;

switch boundaryCond.DefineSource
    case 'T_in + T_out'
        T_source_out    = boundaryCond.T_source_out;
        h_22            = CoolProp.PropsSI('H', 'T', T_source_out, 'P', p_source,  medium_source);
        T_22            = T_source_out;
        m_flow_source   = Q_dot_source / (h_21-h_22);
    case 'T_in + m_flow'
        m_flow_source   = boundaryCond.m_flow_source;
        h_22            = h_21 - Q_dot_source/m_flow_source;
        T_22            = CoolProp.PropsSI('T', 'P', p_source, 'H', h_22,  medium_source);
        T_source_out    = T_22;
    otherwise
        disp('Definition of heat source not implemented')
end
% Discretized temperature profiles

h_evap              = linspace(h(7), h(8), n_states_evap);  % [h_evap_evap(1:end-1), h_evap_sh];
h_source            = linspace(h_22, h_21, n_states_evap);  % [h_source_evap(1:end-1), h_source_sh];

% Temperature Profile -> from 6 to 1

T_evap(1)           = T(7);
T_evap(end)         = T(8);
T_source(1)         = T_22;
T_source(end)	    = T_21;

for i=2:length(h_evap)-1   % parfor
    try
        T_evap(i)         = refpropm_v1('T', 'h', h_evap(i)  , 'p', p_evap, medium, composition);
    catch
        T_evap(i)         = NaN;
    end
    try
        T_source(i)       = CoolProp.PropsSI('T', 'H', h_source(i), 'P', p_source,  medium_source);
    catch
        T_source(i)       = NaN;
    end
end

delta_T_source      = T_source - T_evap;
Q_dot_source_cum    = m_flow*(h_evap-h(7));

% Calculate UA
for i=1:(length(h_evap)-1)
    if ((T_source(i)-T_evap(i)) / (T_source(i+1)-T_evap(i+1)))==1
        delta_T_lm_source(i) = (T_source(i)-T_evap(i));
    else
        delta_T_lm_source(i) = ((T_source(i)-T_evap(i)) - (T_source(i+1)-T_evap(i+1))) / log((T_source(i)-T_evap(i)) / (T_source(i+1)-T_evap(i+1)));
    end
    UA_source(i) = (Q_dot_source_cum(i+1)-Q_dot_source_cum(i)) / delta_T_lm_source(i);
end

UA_source_total = nansum(UA_source); % sum(max(UA_source,0));
delta_T_source_average = Q_dot_source/UA_source_total;
%  |<- End Cycle Model

%% Error message if result is not reasonable
if sum(isnan(T_cond))>5 && show_final_info==1
    display(strcat({'Result not valid: '}, num2str(sum(isnan(T_cond))),{' Points did  not converge in condenser'}))
    fejl(0)
end
if sum(isnan(T_evap))>5 && show_final_info==1
    display(strcat({'Result not valid: '}, num2str(sum(isnan(T_evap))),{' Points did  not converge in evaporator'}))
    fejl(0)
end

%% Sumarizing Variables
% Variables for Plotting
    h_cycle = [h(1) h_cond' h_ihx_hot h_evap h_ihx_cold];

    p_cycle = [p_evap_vector(end) p_cond_vector' p_ihx_hot_vector' p_evap_vector' p_ihx_cold_vector'];
    
% Calculation of remaining state variables
for i=1:length(h)
    try
        s(i) = refpropm_v1('s', 'h', h(i), 'p', p(i), medium, composition);
    catch
        s(i) = NaN;
    end
end
    
%% Post Processing
W_dot_comp  = m_flow*(h_2_adiabat-h(1)); % m_flow*(h(2)-h(1));
W_dot       = W_dot_comp/eta_motor;
COP         = Q_dot_sink_off/W_dot;

Q_dot_motor_loss = W_dot-W_dot_comp;
Q_dot_comp_loss = m_flow * (h_2_adiabat-h(2));

check_EB = W_dot + Q_dot_source - Q_dot_sink_off - Q_dot_motor_loss - Q_dot_comp_loss;

%  Compressor
V_dot_comp_in_h     = V_dot_comp_in * 3600;
V_dot_comp_disp_h   = V_dot_comp_disp * 3600;

p_ratio = p_cond/p_evap;

VHC         = Q_dot_sink_off/V_dot(1);                 % [W/(m3/s)] = [J/m3]
VHC_h       = Q_dot_sink_off/V_dot(1); 

%% Writing results into result structure
% Thermodynamic Cycle
cycle.medium = medium;
cycle.composition = composition;
cycle.mediumString = cycleInput.mediumString;
cycle.h = h;
cycle.s = s;
cycle.p = p;
cycle.T = T;
cycle.p_complete = p_cycle;
cycle.h_complete = h_cycle;
cycle.m_flow        = m_flow;

% Summary
% Performance
summary.medium = medium;
summary.composition = composition;
summary.mediumString = cycleInput.mediumString;
summary.Performance.COP         = COP;
summary.Performance.VHC         = VHC;
summary.Performance.VHC_h       = VHC_h;
summary.Performance.V_dot_comp_in   = V_dot_comp_in;
summary.Performance.V_dot_comp_in_h = V_dot_comp_in_h;
summary.Performance.p_evap      = p_evap;
summary.Performance.p_cond      = p_cond;
summary.Performance.p_ratio     = p_ratio;
summary.Performance.UA_source_total = UA_source_total;
summary.Performance.UA_sink_total   = UA_sink_total;
summary.Performance.UA_sihx_total   = UA_ihx_total;
summary.Performance.m_flow_soure    = m_flow_source;
summary.Performance.m_flow_sink     = m_flow_sink;

% Energy Balance EB
summary.EB.Q_dot_source         = Q_dot_source;
summary.EB.Q_dot_sink           = Q_dot_sink_off;
summary.EB.Q_dot_ihx            = Q_dot_ihx;
summary.EB.Q_dot_motor_loss     = Q_dot_motor_loss;
summary.EB.Q_dot_comp_loss      = Q_dot_comp_loss;
summary.EB.W_dot_comp           = W_dot_comp;
summary.EB.W_dot                = W_dot;
summary.EB.check_EB             = check_EB;

% Temperature Heat Profiles TQ
summary.TQ.Q_dot_source_cum     = Q_dot_source_cum;
summary.TQ.Q_dot_sink_cum       = Q_dot_sink_cum;
summary.TQ.Q_dot_ihx_cum        = Q_dot_ihx_cum;
summary.TQ.T_source             = T_source;
summary.TQ.T_sink               = T_sink;
summary.TQ.delta_T_source       = delta_T_source;
summary.TQ.delta_T_sink         = delta_T_sink;
summary.TQ.delta_T_lm_source    = delta_T_source;
summary.TQ.delta_T_lm_sink      = delta_T_sink;
summary.TQ.delta_T_lm_ihx       = delta_T_ihx;
summary.TQ.T_evap               = T_evap;
summary.TQ.T_cond               = T_cond;
summary.TQ.delta_T_source_average     = delta_T_source_average;
summary.TQ.delta_T_sink_average       = delta_T_sink_average;
summary.TQ.delta_T_pinch_sink_min     = delta_T_pinch_sink_min;
summary.TQ.delta_T_pinch_source_min   = delta_T_pinch_source_min;
summary.TQ.delta_T_pinch_sink   = min(delta_T_sink(1:length(delta_T_sink)-n_states_sub));  % n_states_desup+1:length(delta_T_sink)
summary.TQ.delta_T_pinch_subcool= T(5)-T_sink_in;
summary.TQ.delta_T_pinch_source = min(delta_T_source);
summary.TQ.T_ihx_hot            = T_ihx_hot;
summary.TQ.T_ihx_cold           = T_ihx_cold;

summary.Component.HX.UA_source  = UA_source;
summary.Component.HX.UA_sink    = UA_sink;
summary.Component.HX.UA_ihx     = UA_ihx;
summary.Component.HX.p_evap_vector = p_evap_vector;
summary.Component.HX.p_cond_vector = p_cond_vector;
summary.Component.HX.p_ihx_cold_vector = p_ihx_cold_vector;
summary.Component.HX.p_ihx_hot_vector  = p_ihx_hot_vector;
summary.Component.HX.h_evap     = h_evap;
summary.Component.HX.h_source   = h_source;
summary.Component.HX.h_cond     = h_cond;
summary.Component.HX.h_sink     = h_sink;
summary.Component.HX.h_ihx_hot  = h_ihx_hot;
summary.Component.HX.h_ihx_cold = h_ihx_cold;

summary.Component.Comp.eta_is   = eta_is_comp;
summary.Component.Comp.V_dot_comp_in   = V_dot_comp_in;
summary.Component.Comp.V_dot_comp_in_h = V_dot_comp_in_h;
summary.Component.Comp.V_dot_comp_disp = V_dot_comp_disp;
summary.Component.Comp.V_dot_comp_disp_h = V_dot_comp_disp_h;

%% Exergy analysis (if required)
if strcmp(cycleInput.calcExergy, 'on')==1
   summary.Exergy = HP_ihx_ExergyAnalysis(boundaryCond, cycleInput, componentInput, summary, cycle); 
end

%% Economy calculations (if required)
if strcmp(economyInput.calcInv, 'on')==1
    summary.Investment = HP_ihx_Investment(UA_source, UA_sink, UA_ihx, V_dot_comp_in_h, Q_dot_sink_off,cycleInput, componentInput, economyInput);
    % display('Calculate Investment Cost')
end

if strcmp(economyInput.calcEco, 'on')==1
    summary.Economy = HPEconomy(summary.Investment.TCI, Q_dot_sink_off, Q_dot_source, W_dot, economyInput);
    % display('Calculate Economy')
end

end
