function Economy = HPEconomy(TCI,Q_dot_sink, Q_dot_source, W_dot, economyInput)
%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
% ------------------------------------------------------------------------
% The function calculates the different cost contributions (investment, running cost, ...) 
% and supplies the values in 2 ways:
%   - CF:   Cash flows as annual cash flows [/a]
%   - NPV:  Net present values in which the cash flows are expressed by
%           their values at the time of investment []

%% Capital Recovery Factor
    CRF = economyInput.i_eff*(1+economyInput.i_eff)^economyInput.n_lt / ((1+economyInput.i_eff)^economyInput.n_lt-1); % [-] Cost recovery factor
    
%% Cash flows 
% Fuel cost HP
    % Annual cash flow for fuel cost electricity of heat pump
    CF_FCel         = W_dot * (economyInput.c_el/3600/1000) * economyInput.OH * 3600;                % [/a] 
    % Annual cash flow for fuel cost for heat source of heat pump
    CF_FChsource    = Q_dot_source * (economyInput.c_hsource/3600/1000) * economyInput.OH * 3600;    % [/a]  
    
% Alternative Heat supply
    % Annual cash flow for alternative heat supply
    CF_alt          = Q_dot_sink/economyInput.eta_alt * (economyInput.c_alt/3600/1000) * economyInput.OH * 3600; % [/a] 
    
% Investment Cost 
    CF_TCI          = TCI * CRF; % [/a]           
    
% Cost for O&M as one time cost at time of investment 
    CF_OM           = economyInput.f_OM * TCI * CRF;  % [/a]
    
% Total annual cash flows
    CF_total        = - CF_TCI - CF_OM - CF_FChsource - CF_FCel + CF_alt;
    CF_HG           = CF_TCI + CF_OM + CF_FChsource + CF_FCel;
    c_hg            = CF_HG/(Q_dot_sink * economyInput.OH / 1000000) ;
    
%% Net present values    
% Fuel cost HP
    % Annual cash flow for fuel cost electricity of heat pump
    NPV_FCel        = CF_FCel/CRF;          % [] 
    % Annual cash flow for fuel cost for heat source of heat pump
    NPV_FChsource   = CF_FChsource/CRF;     % []  
    
% Alternative Heat supply
    % Annual cash flow for alternative heat supply
    NPV_alt         = CF_alt/CRF;           % [] 
    
% Investment Cost 
    NPV_TCI         = TCI;                  % []           
    
% Cost for O&M as one time cost at time of investment 
    NPV_OM          = economyInput.f_OM * TCI;  % []

% Total net present value of investment
    NPV_total       = - NPV_TCI - NPV_OM - NPV_FChsource - NPV_FCel + NPV_alt;
    
%% Pay back time
if CF_alt>(CF_FCel-CF_FChsource)
    PBT = NPV_TCI / (CF_alt-CF_FCel-CF_FChsource);   % Simple Payback Time in [a]
else
    % Operating expenses are higher than income -> Infinite PBT
    PBT = Inf;
end
%% Summarize outputs
%  Cash flows
Economy.CF_FCel         = CF_FCel;          % [/a] CF for el of HP 
Economy.CF_FChsource    = CF_FChsource;     % [/a] CF for heat source of HP
Economy.CF_alt          = CF_alt;           % [/a] CF for alternative heat supply
Economy.CF_TCI          = CF_TCI;           % [/a] CF for investment cost compensation
Economy.CF_OM           = CF_OM;            % [/a] CF for O&M cost compensation
Economy.CF_total        = CF_total;         % [/a] Sum of all cash flows
Economy.CF_HGC          = CF_HG;           % [/a] Sum of all cost to produce the supplied heat

%  Net present value
Economy.NPV_FCel        = NPV_FCel;         % [] NPV of el of HP 
Economy.NPV_FChsource   = NPV_FChsource;    % [] NPV of heat source of HP
Economy.NPV_alt         = NPV_alt;          % [] NPV of alternative heat supply
Economy.NPV_TCI         = NPV_TCI;          % [] NPV of investment cost compensation
Economy.TCI_spec        = NPV_TCI/Q_dot_sink/1000; % [/kW] Specific investment cost per unit of supplied heat
Economy.NPV_OM          = NPV_OM;           % [] NPV of O&M cost compensation
Economy.NPV_total       = NPV_total;        % [] Total NPV

%  Pay back time
Economy.PBT             = PBT;              % [years] Simple Payback Time

% Heat generation cost
Economy.c_hg             = c_hg;            % [/MWh] Specific cost for heat generation per unit supplied heat

end