function Investment = HP_std_Investment(UA_source, UA_sink, V_dot_comp_in_h, Q_dot_sink, cycleInput, componentInput, economyInput);
%% ------------------------------------------------------------------------
% The file is part of the numerical models: "Numerical models for the
% design and analysis of heat pumps with zeotropic mixtures". All
% information about copyright, licenses and authors can be found in the
% attached documentation.
% doi: 10.11583/DTU.6825443
% Copyright (c) 2018, B. Zhlsdorf, DTU <bezuhls@mek.dtu.dk>, 03/12/2018
% ------------------------------------------------------------------------
%% Script is used to calculated simplified dimensions and costs for the HP
alpha_sensitivity = 1;
% alpha_sensitivity = alpha_sensitivity_vector(iter_alpha);

n_states_evap = componentInput.HX.n_states_evap;
n_states_sh = componentInput.HX.n_states_sh;
n_states_desup = componentInput.HX.n_states_desup;
n_states_cond = componentInput.HX.n_states_cond;
n_states_sub = componentInput.HX.n_states_sub;
eta_vol_comp = componentInput.Comp.eta_vol_comp;

composition = cycleInput.composition;

%% Defining Inputs
% Heat transfer coefficient

    alpha_evap_pure = 3000;                                             % Evaporation pure fluid
    alpha_evap_mix  = alpha_evap_pure * 0.75 * alpha_sensitivity;       % Evaporation mixture
    alpha_cond_pure = 0.8 * alpha_evap_pure;                            % Condensation pure fluid
    alpha_cond_mix  = alpha_cond_pure * 0.75 * alpha_sensitivity;       % Condensation mixture
    alpha_liq = 0.5 * alpha_evap_pure;                                  % Liquid Phase - primary and secondary side
    alpha_gas = 0.4 * alpha_evap_pure;                                  % Gas Phase - primary side
    
% Flammability
flammables = {'methane', 'ethylene', 'ethane', 'propylen', 'propane', 'DME', 'isobutan', ...
                    'butane', 'ipentane', 'DEE', 'pentane', 'hexane', 'heptane'};

if ~ismember(cell2mat(cycleInput.medium(1,1)),flammables)
    f_flammable = 1.0;
else
    f_flammable = 1.0;
end

%% Sizing HX
% Preallocate space
    U_source = NaN(size(UA_source));                        % [W/m2K]
    U_sink = NaN(size(UA_sink));                            % [W/m2K]

% Source
    for i=1:length(UA_source(:,1))
        if i<n_states_evap
            % Evaporation zone
            if length(composition)==1
                U_source(i) = 1/(1/alpha_evap_pure + 1/alpha_liq);
            else
                U_source(i) = 1/(1/alpha_evap_mix + 1/alpha_liq);
            end
        elseif i>=n_states_evap
            % Superheating zone
            U_source(i) = 1/(1/alpha_gas + 1/alpha_liq);
        end
    end

    A_source = UA_source./U_source;
    A_source_total = sum(A_source);
    
% Sink
    for i=1:length(UA_sink(:,1))
        if i<n_states_sub
            % Subcooling zone
            U_sink(i) = 1/(1/alpha_liq + 1/alpha_liq);
        elseif i>=n_states_sub && i<(n_states_sub+n_states_cond-1)
            % Evaporation zone
            if length(composition)==1
                U_sink(i) = 1/(1/alpha_cond_pure + 1/alpha_liq);
            else
                U_sink(i) = 1/(1/alpha_cond_mix + 1/alpha_liq);
            end
        else
            % Desuperheating zone
            U_sink(i) = 1/(1/alpha_gas + 1/alpha_liq);
        end

    end
    
    A_sink = UA_sink./U_sink;
    A_sink_total = sum(A_sink);
    
%% Purchased Equipment Cost
    PEC_Evap = PEC_HX(A_source_total);
    PEC_Cond = PEC_HX(A_sink_total);
    PEC_Comp = f_flammable * PEC_Compr(V_dot_comp_in_h, eta_vol_comp);
    PEC_total = PEC_Evap + PEC_Cond + PEC_Comp;
    
%% Total Capital Investment Cost
    TCI = PEC_total * economyInput.f_TCI;
    
%% Summarizing Results
Investment.A_sink   = A_sink_total;
Investment.A_source = A_source_total;
Investment.PEC_Evap = PEC_Evap;
Investment.PEC_Cond = PEC_Cond;
Investment.PEC_Comp = PEC_Comp;
Investment.PEC_total = PEC_total;
Investment.TCI = TCI;
Investment.TCI_spec = TCI/Q_dot_sink*1000; % [/kW] Specific investment cost per unit of supplied heat

end
    

    

    
    